// Copyright (c) 2014-2020 NetEase, Inc.
// All right reserved.

library room_kit;

import 'dart:async';
import 'dart:convert';
import 'dart:io';
import 'dart:developer';

import 'package:connectivity/connectivity.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter/foundation.dart';
import 'package:nertc_faceunity/nertc_faceunity.dart';
import 'package:path_provider/path_provider.dart';
import 'package:yunxin_room_kit/room_service.dart';
import 'package:whiteboard_core/whiteboard_core.dart';
import 'package:yunxin_base/yunxin_base.dart';
import 'package:nim/nim.dart';
import 'package:nertc/nertc.dart';
import 'package:nertc_screen_share/nertc_screen_share.dart';
import 'package:flutter_newebview/webview_flutter.dart';
import 'package:permission_handler/permission_handler.dart';

export 'package:nertc/nertc.dart'
    show
        NERtcVideoView,
        NERtcVideoRenderer,
        NERtcRemoteVideoStreamType,
        VideoRendererFactory;
export 'package:yunxin_room_kit/room_service.dart'
    show
        NEResult,
        RoomErrorCode,
        ClientType,
        RoleType,
        JoinRoomInfo,
        NELiveLayout,
        NERoomItem,
        NERoomItemStatus,
        NERoomItemSettings,
        NEPreRoomLiveInfo,
        NERoomScene,
        EventTrackMixin,
        TrackEventName,
        NELiveAuthLevel,
        NERoomItemLiveState,
        NERoomType,
        NEHandsUpStatus,
        SDKConfig,
        DeviceInfo,
        UserProfile;

part 'src/room_kit/auth_service.dart';
part 'src/room_kit/in_room_chatmessage.dart';
part 'src/room_kit/in_room_service.dart';
part 'src/room_kit/in_room_service_listener.dart';
part 'src/room_kit/model/in_room_user_info.dart';
part 'src/room_kit/model/room_live_info.dart';
part 'src/room_kit/invocation_manager.dart';
part 'src/room_kit/module_name.dart';
part 'src/room_kit/pre_room_service.dart';
part 'src/room_kit/model/room_info.dart';
part 'src/room_kit/room_sdk.dart';
part 'src/room_kit/room_service.dart';
part 'src/room_kit/utils/room_utils.dart';
part 'src/room_kit/settings_service.dart';
part 'src/room_kit/value/room_sdk_strings.dart';
part 'src/room_kit/impl/auth_service_impl.dart';
part 'src/room_kit/impl/in_room_data.dart';
part 'src/room_kit/impl/pre_room_service_impl.dart';
part 'src/room_kit/impl/room_sdk_impl.dart';
part 'src/room_kit/impl/room_service_impl.dart';
part 'src/room_kit/impl/in_room_service_impl.dart';
part 'src/room_kit/impl/settings_service_impl.dart';
part 'src/room_kit/controller/in_room_audio_controller.dart';
part 'src/room_kit/controller/room_beauty_controller.dart';
part 'src/room_kit/controller/in_room_chat_controller.dart';
part 'src/room_kit/controller/in_room_cloud_record_controller.dart';
part 'src/room_kit/controller/in_room_live_stream_controller.dart';
part 'src/room_kit/controller/in_room_screen_share_controller.dart';
part 'src/room_kit/controller/in_room_video_controller.dart';
part 'src/room_kit/controller/in_room_whiteboard_controller.dart';
part 'src/room_kit/controller/imp/in_room_cloud_record_controller_impl.dart';
part 'src/room_kit/controller/imp/in_room_whiteboard_controller_impl.dart';
part 'src/room_kit/controller/imp/room_beauty_controller_impl.dart';
part 'src/room_kit/controller/imp/in_room_chat_controller_impl.dart';
part 'src/room_kit/controller/imp/in_room_audio_controller_impl.dart';
part 'src/room_kit/controller/imp/in_room_live_stream_controller_impl.dart';
part 'src/room_kit/controller/imp/in_room_screen_share_controller_impl.dart';
part 'src/room_kit/controller/imp/in_room_video_controller_impl.dart';
part 'src/room_kit/controller/imp/in_room_user_controller_impl.dart';
part 'src/room_kit/controller/in_room_user_controller.dart';
part 'src/room_kit/impl/room_media_service.dart';
part 'src/room_kit/configs/room_configs.dart';
part 'src/room_kit/widget/room_user_video_view.dart';
part 'src/room_kit/http_api_service.dart';
part 'src/room_kit/impl/http_api_service_impl.dart';
part 'src/room_kit/event_track/event_track.dart';
part 'src/room_kit/log/log_service.dart';
