// Copyright (c) 2014-2020 NetEase, Inc.
// All right reserved.

library room_service;

import 'dart:async';
import 'dart:convert';
import 'dart:io';

import 'package:connectivity/connectivity.dart';
import 'package:device_info/device_info.dart';
import 'package:dio/dio.dart' as http;
import 'package:dio/dio.dart';
import 'package:flutter/foundation.dart';
import 'package:flutter/services.dart';
import 'package:yunxin_base/yunxin_base.dart';
import 'package:yunxin_event_track/yunxin_event_track.dart';
import 'package:nim/nim.dart';
import 'package:uuid/uuid.dart';
import 'package:flutter/material.dart';
import 'package:package_info/package_info.dart';
import 'dart:typed_data' show Uint8List;
import 'package:shared_preferences/shared_preferences.dart';

export 'package:nim/nim.dart';

part 'src/room_service/api/api_helper.dart';
part 'src/room_service/api/chatroom_api_helper.dart';
part 'src/room_service/api/http_api_helper.dart';
part 'src/room_service/api/nim_api_helper.dart';
part 'src/room_service/auth/auth_manager.dart';
part 'src/room_service/auth/auth_state.dart';
part 'src/room_service/auth/im_auth_manager.dart';
part 'src/room_service/config/sdk_config.dart';
part 'src/room_service/config/server_config.dart';
part 'src/room_service/device_info.dart';
part 'src/room_service/executor/executors.dart';
part 'src/room_service/executor/http_executor.dart';
part 'src/room_service/executor/nim_executor.dart';
part 'src/room_service/local/sdk_preferences.dart';
part 'src/room_service/model/action_type.dart';
part 'src/room_service/model/av_state.dart';
part 'src/room_service/model/client_type.dart';
part 'src/room_service/model/impl/room_item_impl.dart';
part 'src/room_service/model/join_control_type.dart';
part 'src/room_service/model/live_layout_type.dart';
part 'src/room_service/model/meeting_action.dart';
part 'src/room_service/model/room_item.dart';
part 'src/room_service/model/room_item_setting.dart';
part 'src/room_service/model/room_item_status.dart';
part 'src/room_service/model/room_item_live.dart';
part 'src/room_service/model/meeting_type.dart';
part 'src/room_service/model/member_info.dart';
part 'src/room_service/model/in_room_info.dart';
part 'src/room_service/model/request/anonymous_join_meeting_req.dart';
part 'src/room_service/model/request/close_meeting_req.dart';
part 'src/room_service/model/request/create_meeting_req.dart';
part 'src/room_service/model/request/host_control_req.dart';
part 'src/room_service/model/request/join_meeting_req.dart';
part 'src/room_service/model/request/login_by_token_req.dart';
part 'src/room_service/model/request/login_by_pwd_req.dart';
part 'src/room_service/model/request/login_by_sso_req.dart';
part 'src/room_service/model/request/meeting_members_info_req.dart';
part 'src/room_service/model/request/member_control_req.dart';
part 'src/room_service/model/response/anonymous_join_meeting_res.dart';
part 'src/room_service/model/response/join_meeting_info.dart';
part 'src/room_service/model/response/login_res.dart';
part 'src/room_service/model/response/account_settings.dart';
part 'src/room_service/model/response/result.dart';
part 'src/room_service/remote/base_api.dart';
part 'src/room_service/remote/chatroom/chatroom_enter_api.dart';
part 'src/room_service/remote/chatroom/chatroom_exit_api.dart';
part 'src/room_service/remote/chatroom/chatroom_receive_message_api.dart';
part 'src/room_service/remote/chatroom/chatroom_send_message_api.dart';
part 'src/room_service/remote/http/anonymous_join_meeting_api.dart';
part 'src/room_service/remote/http/cancel_meeting_api.dart';
part 'src/room_service/remote/http/create_meeting_api.dart';
part 'src/room_service/remote/http/delete_meeting_api.dart';
part 'src/room_service/remote/http/edit_meeting_api.dart';
part 'src/room_service/remote/http/fetch_room_link_api.dart';
part 'src/room_service/remote/http/get_config_api.dart';
part 'src/room_service/remote/http/download_beauty_license_api.dart';
part 'src/room_service/remote/http/get_meeting_item_by_id_api.dart';
part 'src/room_service/remote/http/get_meeting_list_by_status_api.dart';
part 'src/room_service/remote/http/join_meeting_api.dart';
part 'src/room_service/remote/http/login_by_token_api.dart';
part 'src/room_service/remote/http/login_by_pwd_api.dart';
part 'src/room_service/remote/http/login_by_sso_api.dart';
part 'src/room_service/remote/http/schedule_meeting_api.dart';
part 'src/room_service/remote/http/get_settings_api.dart';
part 'src/room_service/remote/http/save_settings_api.dart';
part 'src/room_service/remote/http_api.dart';
part 'src/room_service/remote/nim/close_meeting_api.dart';
part 'src/room_service/remote/nim/host_control_api.dart';
part 'src/room_service/remote/nim/im_message_api.dart';
part 'src/room_service/remote/nim/meeting_info_api.dart';
part 'src/room_service/remote/nim/member_control_api.dart';
part 'src/room_service/remote/nim/online_status_api.dart';
part 'src/room_service/remote/nim/server_control_api.dart';
part 'src/room_service/remote/nim_api.dart';
part 'src/room_service/remote/nim_passthrough_api.dart';
part 'src/room_service/remote/nim_stream_api.dart';
part 'src/room_service/repository/auth_repository.dart';
part 'src/room_service/repository/chatroom_repository.dart';
part 'src/room_service/repository/global_error_repository.dart';
part 'src/room_service/repository/in_room_repository.dart';
part 'src/room_service/repository/room_repository.dart';
part 'src/room_service/repository/pre_room_repository.dart';
part 'src/room_service/repository/service_repository.dart';
part 'src/room_service/repository/settings_repository.dart';
part 'src/room_service/strings.dart';
part 'src/room_service/user_profile.dart';
part 'src/room_service/model/hands_up.dart';
part 'src/room_service/model/whiteboard_status.dart';
part 'src/room_service/model/room_item_live_status.dart';
part 'src/room_service/repository/live_repository.dart';
part 'src/room_service/remote/nim/start_live_api.dart';
part 'src/room_service/remote/nim/update_live_api.dart';
part 'src/room_service/remote/nim/stop_live_api.dart';
part 'src/room_service/model/role_type.dart';
part 'src/room_service/model/live_level.dart';
part 'src/room_service/remote/nim/modify_nickname_api.dart';
part 'src/room_service/model/request/modify_nickname_req.dart';
part 'src/room_service/remote/chatroom/chatroom_update_role_api.dart';
part 'src/room_service/module_name.dart';
part 'src/room_service/model/request/generic_http_api_req.dart';
part 'src/room_service/model/room_scene.dart';
part 'src/room_service/model/room_role_configuration.dart';
part 'src/room_service/event_track/event_track.dart';


/// http://doc.hz.netease.com/pages/viewpage.action?pageId=251213353
class RoomErrorCode {

  /// 通用失败
  static const int failed = -1;

  /// 接口调用失败，原因为参数校验不通过
  static const int paramError = -2;

  /// 接口调用失败，原因为SDK未登录
  static const int notLogin = -3;

  /// 网络错误
  static const int networkError = -4;

  /// 当前存在未结束的房间
  static const int alreadyInRoom = -5;

  /// IM复用登录失败，原因是 IM-SDK 未登录
  static const int loginErrorIMNotLogin = -6;

  /// IM复用登录失败，原因是 IM-SDK 账号与 RoomKit 账号不匹配
  static const int loginErrorIMAccountNotMatch = -7;

  /// IM复用匿名登录失败，原因是不支持
  static const int loginErrorAnonymousLoginNotSupport = -8;

  /// 取消
  static const int cancelled = -9;

  /// 服务端错误码
  static const int success = 200;
  static const int paramsError = 300;
  static const int paramsNull = 301;
  static const int serverInnerError = 302;
  static const int unauthorized = 401;
  static const int forbidden = 403;
  static const int notFound = 404;
  static const int netTimeout = 408;
  static const int netError = 415;
  static const int serverError = 501;
  static const int versionNotCompatible = 513;
  static const int allocationAccountFail = 1000;
  static const int fetchAccountFail = 1001;
  static const int controlNotAllocationAccount = 1002;
  static const int phoneAlreadyRegister = 1003;
  static const int phoneNotRegister = 1004;
  static const int tokenError = 1005;
  static const int verifyError = 1006;
  static const int loginPasswordError = 1007;
  static const int passwordError = 1008;
  static const int tvAccountNotExist = 1009;
  static const int illegalPhone = 1010;
  static const int smsCodeOverLimit = 1011;
  static const int verifyInvalid = 1012;
  static const int accountNotExist = 1014;
  static const int roomNotExist = 2000;
  static const int roomMemberOverLimit = 2001;
  static const int roomAlreadyExist = 2002;
  static const int nickError = 2003;
  static const int memberVideoError = 2004;
  static const int memberAudioError = 2005;
  static const int roomCidIsEmpty = 2006;
  static const int memberNotHasRoomId = 2007;
  static const int notJoinRoomCannotKick = 2008;
  static const int screenShareOverLimit = 2009;
  static const int notHostPermission = 2100;
  static const int memberNotInRoom = 2101;
  static const int controlCodeError = 2102;
  static const int roomPasswordError = 2014;
  static const int roomPasswordNotPresent = 2018;
  static const int audioNeedHandsUp = 2108;
  static const int allowSelfAudioOn = 2110;
  static const int audioAlreadyOn = 2111;
  static const int roomNotInProgress = 2200;
  static const int notSupport = 3003;

  static String getMsg(String? msg, [String? defaultTips]) {
    if (msg == null || msg.isEmpty) {
      return (defaultTips == null || defaultTips.isEmpty) ? '网络连接失败，请检查你的网络连接！' : defaultTips;
    }
    return msg;
  }
}
