// Copyright (c) 2014-2020 NetEase, Inc.
// All right reserved.
part of room_kit;

/// 用于在完成SDK登录鉴权后，查询当前已登录账号的基本信息，如个人房间号信息
/// 通过[NERoomKit.getAuthService]获取账号服务的实例
abstract class NEAuthService {
  /// 登录鉴权。在已登录状态下可以创建和加入房间，但在未登录状态下只能加入房间
  ///
  /// * [accountId]   登录accountId
  /// * [token]     登录令牌
  Future<NEResult<void>> loginWithToken(String accountId, String token);

  /// 登录鉴权。在已登录状态下可以创建和加入房间，但在未登录状态下只能加入房间
  ///
  /// * [username]   登录账号
  /// * [password]   登录密码
  Future<NEResult<void>> loginWithAccount(String username, String password);

  /// 登录鉴权。在已登录状态下可以创建和加入房间，但在未登录状态下只能加入房间
  ///
  /// * [ssoToken]  登录SSO令牌
  Future<NEResult<void>> loginWithSSOToken(String ssoToken);

  ///用户自动登录
  Future<NEResult<void>> tryAutoLogin();

  /// 注册登录状态监听器
  ///
  /// [listener] 监听器.
  void addAuthListener(NERoomAuthListener listener);

  /// 注销登录状态监听器
  ///
  /// [listener] 监听器.
  void removeAuthListener(NERoomAuthListener listener);

  /// 注销当前已登录的账号
  Future<NEResult<void>> logout();

  /// 账号信息
  NEAccountInfo? getAccountInfo();
}

class NEAccountInfo {
  /// 个人房间号，该房间号可在创建房间时使用
  final String roomId;

  /// 个人房间短号，该房间号可在创建房间时使用
  final String? shortRoomId;

  /// 账号ID
  final String accountId;

  /// 账户名
  final String accountName;

  NEAccountInfo(this.accountId, this.accountName, this.roomId, this.shortRoomId);

  Map<String, dynamic> toJson() => {
        'roomId': roomId,
        if (shortRoomId != null) 'shortRoomId': shortRoomId,
        'accountId': accountId,
        'accountName': accountName,
      };
}
