// Copyright (c) 2014-2021 NetEase, Inc.
// All right reserved.

part of room_kit;

/// 房间直播控制器
class _NEInRoomLiveStreamControllerImpl extends NEInRoomLiveStreamController with _RoomAloggerMixin {

  final _InRoomData roomData;

  _NEInRoomLiveStreamControllerImpl(this.roomData);

  @override
  Future<NEResult<NEInRoomLiveInfo>> startLiveStream(NEInRoomLiveInfo live) async {
    apiLogger.i('startLiveStream');
    if (live is _InRoomLiveInfo) {
      final updated = _InRoomLiveInfo(roomData, live.delegate.copy());
      updated.userList = live.userList;
      updated.roomId = roomData.roomId;
      final result = await LiveRepository.startLive(updated.delegate);
      return NEResult(code: result.code, msg: result.msg,
          data: result.data != null ? _InRoomLiveInfo(roomData, result.nonNullData) : null);
    }
    return NEResult(code: RoomErrorCode.paramsError);
  }

  @override
  Future<NEResult<void>> stopLiveStream() {
    apiLogger.i('startLiveStream');
    return LiveRepository.stopLive(roomData.roomId);
  }

  @override
  Future<NEResult<NEInRoomLiveInfo>> updateLiveStream(NEInRoomLiveInfo live) async {
    apiLogger.i('startLiveStream');
    if (live is _InRoomLiveInfo) {
      final updated = _InRoomLiveInfo(roomData, live.delegate.copy());
      updated.userList = live.userList;
      updated.roomId = roomData.roomId;
      final result = await LiveRepository.updateLive(updated.delegate);
      return NEResult(code: result.code, msg: result.msg,
          data: result.data != null ? _InRoomLiveInfo(roomData, result.nonNullData) : null);
    }
    return NEResult(code: RoomErrorCode.paramsError);
  }

  @override
  NEInRoomLiveInfo? getLiveStreamInfo() {
    final liveInfo = roomData.roomInfo?.liveStreamInfo;
    return liveInfo != null ? liveInfo.copy() : null;
  }

  @override
  bool isLiveStreamEnabled() {
    return roomData.roomInfo?.liveStreamInfo?.enable == true;
  }
}
