// Copyright (c) 2014-2020 NetEase, Inc.
// All right reserved.

part of room_kit;

///房间内音频控制器.
abstract class NEInRoomAudioController {
  /// 开启/关闭本地语音采集以及发送
  // Future<NEResult<void>> enableMyAudio(bool enable);

  ///主持人将所有与会者静音, [allowUnmuteSelf] true：允许取消自己静音；反之，不允许
  Future<NEResult<void>> muteAllParticipantsAudio(bool allowUnmuteSelf);

  /// 主持人取消全体静音
  Future<NEResult<void>> unmuteAllParticipantsAudio();

  ///主持人将与会者的音频静音
  Future<NEResult<void>> muteParticipantAudio(String userId, bool isMute);

  ///设置自己音频静音或者开启,[isMute] true: 关闭/false：开启
  Future<NEResult<void>> muteMyAudio(bool isMute);

  ///同步音频状态
  Future<NEResult<void>> syncLocalAudioStatus();

  /// 查询本地是否可以开启音频
  bool canUnmuteMyAudio();

  ///查询当前发言者的扬声器状态
  Future<bool> isMyLoudSpeakerOn();

  ///设置是否可以切换当前房间的音频输出源时是否打开扬声器，[on] true: 打开/false：关闭
  Future<NEResult<void>> setMyLoudSpeakerOn(bool on);

  /// 设置是否音频采集静音
  Future<NEResult<void>> setRecordDeviceMute(bool isMute);

  /// 获取音频采集设备是否静音
  Future<bool> isRecordDeviceMute();

  /// 设置是否音频播放静音
  Future<NEResult<void>> setPlayoutDeviceMute(bool isMute);

  /// 获取当前音频播放设备是否静音
  Future<bool> isPlayoutDeviceMute();

  ///开启或关闭耳返
  ///
  /// [enabled] true:开启,false:关闭
  /// [volume] 耳返音量 [0 - 100]（默认 100）
  Future<NEResult<void>> enableEarBack(bool enabled, int volume);

  ///设置耳返音量
  ///
  /// [volume] 耳返音量 [0 - 100]（默认 100）
  Future<NEResult<void>> setEarBackVolume(int volume);

  ///查询是否启用与会者加入房间时自动使其静音
  bool isMuteOnEntryOn();

  /// 订阅会议内某一音频流
  ///
  /// * [userId] 订阅或者取消订阅的id
  Future<NEResult<void>> subscribeRemoteAudioStream(String userId);

  /// 取消订阅会议内某一音频流
  ///
  /// * [userId] 用户id
  Future<NEResult<void>> unsubscribeRemoteAudioStream(String userId);

  /// 订阅会议内全部音频流
  ///
  Future<NEResult<void>> subscribeAllRemoteAudioStream();

  /// 取消订阅会议内全部音频流
  ///
  Future<NEResult<void>> unsubscribeAllRemoteAudioStream();

  /// 开始音频 dump
  Future<NEResult<void>> startAudioDump();

  /// 结束音频dump
  Future<NEResult<void>> stopAudioDump();

  /// 启用说话者音量提示。该方法允许 SDK 定期向 App 反馈当前谁在说话以及说话者的音量
  ///
  /// [enable] 是否启用说话者音量提示
  /// [interval] 指定音量提示的时间间隔，单位为毫秒。必须设置为 100 毫秒的整数倍值
  Future<NEResult<void>> enableAudioVolumeIndication(bool enable, int interval);

  /// 调节录音音量
  /// 加入频道前后都可以调用
  /// 调节范围为：[0~400]
  /// 0: 静音
  /// 100: 原始音量 (默认)
  /// 400: 最大可为原始音量的 4 倍(自带溢出保护)
  ///
  /// [volume] 调节的音量值。
  Future<NEResult<void>> adjustRecordingSignalVolume(int volume);

  /// 调节播放音量
  /// 加入频道前后都可以调用
  /// 调节范围为：[0~400]
  /// 0: 静音
  /// 100: 原始音量 (默认)
  /// 400: 最大可为原始音量的 4 倍(自带溢出保护)
  ///
  /// [volume] 调节的音量值。
  Future<NEResult<void>> adjustPlaybackSignalVolume(int volume);

  /// 举手
  Future<NEResult<void>> raiseMyHand();

  /// 取消举手
  Future<NEResult<void>> lowerMyHand();

    /// 将某个用户的手放下
  Future<NEResult<void>> lowerHand(String userId);

  /// 把所有的用户手放下
  Future<NEResult<void>> lowerAllHands();

  // ///配置音频控制器默认参数
  // Future<void> setUp() async { }

//
// ///
// ///查询是否可以切换当前房间的音频输出源
// bool canSwitchAudioOutput();
//
// /// 获取当前使用的音频输出设备
// /// [NERtcAudioDevice]
// int getMyCurrentAudioDevice();
//
// ///查询用户是否可以自己取消音频静音
// bool canUnmuteMyAudio();
//
//
// bool isMyHeadsetOn();
//
// ///查询当前用户的音频是否被静音
// bool isMyAudioMuted();

// /// 批量订阅会议内音频流
// ///
// /// * [userIds] 订阅或者取消订阅的id列表
// /// * [subscribe] true：订阅， false：取消订阅
// Future<NEResult<List<String>>> subscribeRemoteAudio(List<String> userIds, bool subscribe);

}
