// Copyright (c) 2014-2021 NetEase, Inc.
// All right reserved.
part of room_kit;
/// 控制房间内自己、其他人的视频状态
abstract class NEInRoomVideoController {

  /// 获取当前焦点视频用户ID
  String? getPinnedUserId();

  /// 主持人取消设置/设置对应用户为焦点视频
  Future<NEResult<void>> pinVideo(String userId, bool on);

  /// 查询对应用户是否被设置为焦点视频
  bool isUserPinned(String userId);

  /// 主持人请求房间内其他用户开启视频
  Future<NEResult<void>> askParticipantStartVideo(String userId);

  /// 主持人关闭房间内其他用户的视频
  Future<NEResult<void>> stopParticipantVideo(String userId);

  /// 查询本地视频是否关闭
  bool isMyVideoMuted();

  /// 查询本地是否可以开启视频
  bool canUnmuteMyVideo();

  /// 开启/关闭本地视频
  Future<NEResult<void>> muteMyVideo(bool mute);

  /// 旋转本地视频
  ///bool rotateMyVideo(int rotation);

  /// 查询本地摄像头设备
  ///List<CameraDevice> getCameraDeviceList();

  /// 获取当前所使用的摄像头ID
  // String getSelectedCameraId();

  /// 当前使用的是后置摄像头
  // bool isBackCamera(String cameraId);

  /// 当前使用前置摄像头
  // bool isFrontCamera(String cameraId);

  /// 切换至对应的摄像头
  Future<bool> switchCamera();

  /// 查询本地是否可以切换摄像头
  bool canSwitchCamera();

  ///同步视频状态
  Future<NEResult<void>> syncLocalVideoStatus();

  /// 订阅房间内某一视频流
  ///
  /// * [userId] 用户id
  /// * [streamType] [NERtcRemoteVideoStreamType.high] 大流/ [NERtcRemoteVideoStreamType.low]小流
  Future<NEResult<void>> subscribeRemoteVideoStream(String userId, int streamType);

  /// 取消订阅房间内某一视频流
  ///
  /// * [userId] 用户id
  Future<NEResult<void>> unsubscribeRemoteVideoStream(String userId);

  /// 订阅房间内某一视频辅流
  ///
  /// * [userId] 用户id
  Future<NEResult<void>> subscribeRemoteVideoSubStream(String userId);

  /// 取消订阅房间内某一视频辅流
  ///
  /// * [userId] 用户id
  Future<NEResult<void>> unsubscribeRemoteVideoSubStream(String userId);

  /// 添加为用户视频画布
  Future<NEResult<void>> attachRendererToUserVideoStream(NERtcVideoRenderer renderer, String userId);

  /// 添加为用户辅流视频画布
  Future<NEResult<void>> attachRendererToUserVideoSubStream(NERtcVideoRenderer renderer, String userId);

}