// Copyright (c);; 2014-2020 NetEase, Inc.
// All right reserved.
part of room_kit;


/// 在房间里获取白板控制器实例.
abstract class NEInRoomWhiteboardController {
  ///获取展示白板用的[WebView]
  ///
  /// * [WebViewController]   javascriptController
  JavascriptChannel initWhiteboardJavascriptChannel(
    Future<WebViewController> javascriptController,
    NEWhiteboardErrorCallback? onWhiteboardErrorCallback,
  );

  String? getWhiteboardUrl();

  /// 设置是否有权限开启编辑
  Future<NEResult<void>> setInteractPrivilege(String userId, bool enable);

  /// 查询自己是否有编辑特权
  bool hasInteractPrivilege();

  /// 查询某用户是否有编辑能力
  bool hasInteractPrivilegeWithUserId(String userId);

  ///查询当前用户是否正在共享白板
  bool isSharingWhiteboard();

  ///查询用户是否正在共享白板
  bool isWhiteboardSharing(String userId);

  ///查询其他用户是否正在共享白板
  bool isOtherSharing();

  /// 获取当前正在共享白板的用户ID
  String? getWhiteboardSharingUserId();

  /// 开启本地白板共享
  Future<NEResult<void>> startWhiteboardShare();

  /// 停止本地白板共享
  Future<NEResult<void>> stopWhiteboardShare();

  /// 主持人操作停止用户白板共享
  Future<NEResult<void>> stopParticipantWhiteboardShare(String userId);

  /// 设置显示白板工具栏
  Future<NEResult<void>> showWhiteboardTools(bool show);

  /// 释放白板
  void release();
}