// Copyright (c); 2014-2020 NetEase, Inc.
// All right reserved.

part of room_kit;

/// 在房间里获取美颜控制器实例.
abstract class NERoomBeautyController with _RoomAloggerMixin{
  late NERtcFaceUnityEngine _beautyEngine;

  ///是否开启美颜状态
  late bool _isEnabled = false;

  NERoomBeautyController() {
    _beautyEngine = NERtcFaceUnityEngine();
  }

  Future<NEResult<void>> enableBeauty(bool enable) async {
    apiLogger.i('enableBeauty enable=$enable');
    ///开启美颜的执行返回，成功/失败
    NEResult enableBeautyResult;
    if (enable ==_isEnabled) return NEResult.success();
    if (enable) {
      final license = (await SettingsRepository.getBeautyLicenseApi());
      final hasLicense = license != null && license.data != null;
      if (hasLicense) {
        await _beautyEngine.create(beautyKey: license!.data!);
      }
      _isEnabled = hasLicense;
      enableBeautyResult = hasLicense
          ? NEResult.success()
          : NEResult(code: RoomErrorCode.notSupport);
    } else {
      _isEnabled = false;
      release();
      enableBeautyResult = NEResult.success();
    }

    return enableBeautyResult;
  }

  Future<NEResult<int>> getBeautyFaceValue() async {
    var isBeautyFaceSupported = SettingsRepository.isBeautyFaceSupported();
    if (!isBeautyFaceSupported) {
      return NEResult(code: RoomErrorCode.notSupport);
    }
    return await SettingsRepository.getBeautyFaceValue();
  }

  bool isBeautyEnabled() => _isEnabled;

  Future<NEResult<void>> setBeautyFaceValue(int value,
      {bool save = true}) async {
    apiLogger.i('setBeautyFaceValue $value');
    var isBeautySupported = SettingsRepository.isBeautyFaceSupported();
    var result = NEResult.success();
    if (!isBeautySupported) {
      return NEResult(code: RoomErrorCode.notSupport);
    }
    if (SettingsRepository.beautyLevelCache == value) return NEResult.success();
    apiLogger.i(' real setBeautyFaceValue $value');
    if (save) {
      result = await SettingsRepository.setBeautyFaceValue(value);
    }
    var beautyResult = await _beautyEngine.setBeautyParams(beautyLevels[value]);
    if (beautyResult != 0){
      commonLogger.i('set beauty params fail value $value ');
      return NEResult(code: RoomErrorCode.failed);
    }
    return result;
  }

  void release() {
    _beautyEngine.release();
  }

  /// 默认美颜配置表,等级对应数组
  final beautyLevels = List<NERtcFaceUnityParams>.generate(11, (index) {
    var level = index * 0.1;
    return NERtcFaceUnityParams()
      ..beautyLevel = index
      ..filterLevel = level
      ..colorLevel = level
      ..redLevel = level
      ..blurLevel = level
      ..eyeBright = level;
  });
}

/// 在房间里获取美颜控制器实例.
abstract class NEInRoomBeautyController extends NERoomBeautyController {}

abstract class NEPreRoomBeautyController extends NERoomBeautyController {
  ///创建美颜预览View
  Future<NERtcVideoRenderer?> createVideoPreview();
}
