// Copyright (c) 2014-2021 NetEase, Inc.
// All right reserved.

part of room_kit;

class _InRoomData {

  final JoinRoomInfo joinRoomInfo;

  /// 入会需要传入的option
  final NERoomOptions options;

  _InRoomData(this.joinRoomInfo, this.options);

  _RoomInfo? roomInfo;

  late _InRoomUser self;

  /// userId---> member
  final Map<String, _InRoomUser> userId2User = <String, _InRoomUser>{};

  /// roomID---> member
  final Map<int, _InRoomUser> rtcId2User = <int, _InRoomUser>{};

  /// 等待获取信息的用户列表
  final pendingUser = <int>{};

  String get roomId => joinRoomInfo.meetingId;
  /// 匿名入会，accountId为空
  String get myUserId => UserProfile.accountId ?? '';

  String? getUserName(String userId) => userId2User[userId]?.displayName;
}

class _RoomInfo extends NERoomInfo {

  /// 预定成功后， 服务器生成唯一id
  @override
  final int uniqueId;

  /// 会议ID
  @override
  final String roomId;

  /// 会议短号ID
  @override
  final String? shortRoomId;

  /// Sip会议ID
  @override
  final String? sipCid;

  /// 会议类型
  @override
  final int type;

  /// 会议主题
  @override
  final String subject;

  /// 会议密码
  @override
  final String? password;

  /// 会议预约开始时间
  @override
  final int scheduleStartTime;

  /// 会议预约结束时间
  @override
  final int scheduleEndTime;

  final DualValueNotifier<bool> audioAllMuteNotifier =
  DualValueNotifier(false);
  /// 是否有全体禁音，0：否，1：有
  @override
  bool get audioAllMute => audioAllMuteNotifier.value;

  set audioAllMute(bool value) => audioAllMuteNotifier.value = value;

  final DualValueNotifier<bool> lockedNotifier =
  DualValueNotifier(false);
  /// 会议加入控制类型:，1：允许任何人直接加入，2：不允许任何人加入
  /// [JoinControlType]
  @override
  bool get locked => lockedNotifier.value;

  set locked(bool value) => lockedNotifier.value = value;

  final DualValueNotifier<String> hostUserIdNotifier =
  DualValueNotifier('');
  /// 主持人，视频会议帐号id
  @override
  String get hostUserId => hostUserIdNotifier.value;

  set hostUserId(String value) => hostUserIdNotifier.value = value;

  final DualValueNotifier<String?> pinnedUserIdNotifier = DualValueNotifier(null);
  /// 焦点视频角色，视频会议帐号id
  @override
  String? get pinnedUserId => pinnedUserIdNotifier.value;

  set pinnedUserId(String? value) => pinnedUserIdNotifier.value = value;

  final DualValueNotifier<String?> screenSharingUserIdNotifier = DualValueNotifier(null);
  /// 屏幕共享者
  @override
  String? get screenSharingUserId => screenSharingUserIdNotifier.value;

  set screenSharingUserId(String? value) => screenSharingUserIdNotifier.value = value;

  ////////////////////// whiteboard start ///////////////////////////////////////
  /// [_WhiteboardSharingUserInfo]
  final DualValueNotifier<_WhiteboardSharingUserInfo>
      whiteboardSharingUserInfoNotifier = DualValueNotifier(
    _WhiteboardSharingUserInfo(
      whiteboardShareStatus: NERoomWhiteBoardShareStatus.end,
    ),
  );

  set whiteboardSharingUserInfo(_WhiteboardSharingUserInfo userInfo) {
    whiteboardSharingUserInfoNotifier.value = userInfo;
  }

  /// 白板共享者
  @override
  String? get whiteboardSharingUserId =>
      whiteboardSharingUserInfoNotifier.value.sharerUserId;

  /// 白板拥有者imAccid
  String? get whiteboardOwnerImAccid =>
      whiteboardSharingUserInfoNotifier.value.sharerImAccId;

  /// 白板操作者
  int? get whiteboardOwnerOperatorRtcId =>
      whiteboardSharingUserInfoNotifier.value.operatorRtcId;

  ////////////////////// whiteboard end ///////////////////////////////////////

  final DualValueNotifier<NERoomItemSettings?> settingsNotifier = DualValueNotifier(null);
  /// 会议设置
  @override
  NERoomItemSettings? get settings => settingsNotifier.value;

  set settings(NERoomItemSettings? value) => settingsNotifier.value = value;

  final DualValueNotifier<_InRoomLiveInfo?> liveStreamInfoNotifier = DualValueNotifier(null);
  /// 会议直播设置
  @override
  _InRoomLiveInfo? get liveStreamInfo => liveStreamInfoNotifier.value;

  set liveStreamInfo(_InRoomLiveInfo? info) => liveStreamInfoNotifier.value = info;

  @override
  List<NEInRoomUserInfo> get userList => userId2User.values.toList();

  final Map<String, _InRoomUser> userId2User;

  final Map<int, _InRoomUser> rtcId2User;

  _RoomInfo({
    required this.uniqueId,
    required this.roomId,
    required this.shortRoomId,
    required this.sipCid,
    required this.type,
    required this.subject,
    required this.password,
    required this.scheduleStartTime,
    required this.scheduleEndTime,
    _InRoomLiveInfo? liveStreamInfo,
    required this.userId2User,
    required this.rtcId2User,
  }) {
    this.liveStreamInfo = liveStreamInfo;
  }

}

class _WhiteboardSharingUserInfo {
  String? sharerUserId;
  int? sharerRtcId;
  int? operatorRtcId;
  String? sharerImAccId;
  NERoomWhiteBoardShareStatus whiteboardShareStatus;

  _WhiteboardSharingUserInfo(
      {this.sharerUserId,
      this.sharerRtcId,
      this.operatorRtcId,
      this.sharerImAccId,
      required this.whiteboardShareStatus});

  @override
  bool operator ==(Object other) {
    return other is _WhiteboardSharingUserInfo &&
        other.runtimeType == runtimeType &&
        other.sharerUserId == sharerUserId &&
        other.sharerRtcId == sharerRtcId &&
        other.operatorRtcId == operatorRtcId &&
        other.whiteboardShareStatus == whiteboardShareStatus &&
        other.sharerImAccId == sharerImAccId;
  }

  @override
  int get hashCode => hashValues(sharerUserId, sharerRtcId, operatorRtcId,
      sharerImAccId, whiteboardShareStatus);
}

class _InRoomRaiseHandDetail extends NERaiseHandDetail {

  @override
  int status;

  @override
  int timestamp;

  _InRoomRaiseHandDetail(this.status, this.timestamp);

  _InRoomRaiseHandDetail.down(): this(NEHandsUpStatus.down, -1);

  @override
  bool operator ==(Object other) =>
      identical(this, other) ||
          other is _InRoomRaiseHandDetail &&
              runtimeType == other.runtimeType &&
              (status == other.status ||
                  (status != NEHandsUpStatus.up && other.status != NEHandsUpStatus.up));

  @override
  int get hashCode => status.hashCode;

  @override
  String toString() {
    if (status == NEHandsUpStatus.up) return 'up';
    if (status == NEHandsUpStatus.agree) return 'agree';
    if (status == NEHandsUpStatus.downByHost) return 'downByHost';
    return 'down';
  }
}

class _InRoomUser extends NEInRoomUserInfo {
  /// 房间用户id
  @override
  final String userId;
  /// 映射音视频房间成员uid
  final int rtcId;

  bool isOnline = false;

  late DualValueNotifier<String> displayNameNotifier;
  /// 房间用户昵称
  @override
  String get displayName => displayNameNotifier.value;

  set displayName(String value) => displayNameNotifier.value = value;

  final DualValueNotifier<_InRoomRaiseHandDetail> raiseHandDetailNotifier =
      DualValueNotifier(_InRoomRaiseHandDetail.down());
  /// 当前举手状态
  @override
  _InRoomRaiseHandDetail get raiseHandDetail => raiseHandDetailNotifier.value;

  set raiseHandDetail(_InRoomRaiseHandDetail value) => raiseHandDetailNotifier.value = value;

  final DualValueNotifier<bool> screenSharingNotifier = DualValueNotifier(false);
  @override
  bool get isScreenSharing => screenSharingNotifier.value;
  set isScreenSharing(bool value) => screenSharingNotifier.value = value;

  final DualValueNotifier<NERoomWhiteBoardShareStatus> whiteBoardSharingNotifier = DualValueNotifier(NERoomWhiteBoardShareStatus.end);

  @override
  bool get isWhiteBoardSharing =>
      whiteBoardSharingNotifier.value == NERoomWhiteBoardShareStatus.start;

  set whiteBoardShareStatus(NERoomWhiteBoardShareStatus status) =>
      whiteBoardSharingNotifier.value = status;

  NERoomWhiteBoardShareStatus get whiteBoardShareStatus =>
      whiteBoardSharingNotifier.value;

  /// 客户端类型
  @override
  final int clientType;

  /// 角色身份，1成员，2主持人，3管理员，4隐藏
  /// [RoleType]
  @override
  final int roleType;

  /// 成员标签，自定义，最大长度50
  @override
  final String? tag;

  final DualValueNotifier<NERoomVideoStatus> videoStatusNotifier =
      DualValueNotifier(NERoomVideoStatus.off);
  /// 视频状态
  @override
  NERoomVideoStatus get videoStatus => videoStatusNotifier.value;

  set videoStatus(NERoomVideoStatus value) {
    /// 已经是视频打开状态，忽略打开请求
    if (value == NERoomVideoStatus.waitingOpen
        && videoStatus == NERoomVideoStatus.on) {
      return;
    }
    videoStatusNotifier.value = value;
  }

  final DualValueNotifier<NERoomAudioStatus> audioStatusNotifier =
  DualValueNotifier(NERoomAudioStatus.off);
  /// 音频状态
  @override
  NERoomAudioStatus get audioStatus => audioStatusNotifier.value;

  set audioStatus(NERoomAudioStatus value) {
    /// 已经是音频打开状态，忽略打开请求
    if (value == NERoomAudioStatus.waitingOpen
        && audioStatus == NERoomAudioStatus.on) {
      return;
    }
    audioStatusNotifier.value = value;
  }

  final DualValueNotifier<bool> pinnedNotifier = DualValueNotifier(false);
  @override
  bool get isPinned => pinnedNotifier.value;
  set isPinned(bool value) => pinnedNotifier.value = value;

  final DualValueNotifier<NERoomWhiteBoardInteractionStatus>
      whiteBoardInteractionStatusNotifier =
      DualValueNotifier(NERoomWhiteBoardInteractionStatus.close);

  /// 白板互动权限
  @override
  NERoomWhiteBoardInteractionStatus get whiteBoardInteractStatus =>
      whiteBoardInteractionStatusNotifier.value;

  set whiteBoardInteractStatus(NERoomWhiteBoardInteractionStatus value) =>
      whiteBoardInteractionStatusNotifier.value = value;

  /// G2层该用户的音频状态
  bool audioStartedOnRtcSide = false;

  /// 用户的音频状态在Rtc与会议服务出现不一致
  bool get audioStatusInconsistentWithRtc => 
    (audioStartedOnRtcSide && audioStatus != NERoomAudioStatus.on)
    || (!audioStartedOnRtcSide && audioStatus == NERoomAudioStatus.on);

  /// G2层该用户的视频状态
  bool videoStartedOnRtcSide = false;

  /// 用户的视频状态在Rtc与会议服务出现不一致
  bool get videoStatusInconsistentWithRtc => 
    (videoStartedOnRtcSide && videoStatus != NERoomVideoStatus.on)
    || (!videoStartedOnRtcSide && videoStatus == NERoomVideoStatus.on);

  _InRoomUser({
    required this.userId,
    required this.rtcId,
    required this.clientType,
    required this.roleType,
    required this.tag,
    required this.isOnline,
    required String displayName,
    required NERoomAudioStatus audioStatus,
    required NERoomVideoStatus videoStatus,
    required bool isPinned,
    required bool isScreenSharing,
    required NERoomWhiteBoardShareStatus whiteBoardShareStatus,
    required _InRoomRaiseHandDetail raiseHandDetail,
    required NERoomWhiteBoardInteractionStatus whiteBoardInteractStatus,
  }) {
    displayNameNotifier = DualValueNotifier(displayName);
    this.displayName = displayName;

    this.audioStatus = audioStatus;
    this.videoStatus = videoStatus;
    this.isPinned = isPinned;
    this.isScreenSharing = isScreenSharing;
    this.whiteBoardShareStatus = whiteBoardShareStatus;
    this.raiseHandDetail = raiseHandDetail;
    this.whiteBoardInteractStatus = whiteBoardInteractStatus;
  }

}

class OnlineStatus {
  /// 占位
  static const int placeholder = 1;
  /// 在线
  static const int online = 2;
}

class _InRoomLiveInfo extends NEInRoomLiveInfo {

  final _InRoomData? roomData;
  final NERoomItemLive delegate;
  List<String>? _userList;

  _InRoomLiveInfo(
    this.roomData,
    this.delegate,
  ) : super._();

  _InRoomLiveInfo.empty()
      : roomData = null,
        delegate = NERoomItemLive(),
        super._();

  set roomId(String value) => delegate.roomId = value;

  @override
  bool get enable => delegate.enable;

  @override
  String? get title => delegate.title;

  @override
  set title(String? title) => delegate.title = title;

  @override
  String? get password => delegate.password;

  /// 直播密码
  @override
  set password(String? password) => delegate.password = password;

  /// 直播状态
  @override
  NERoomItemLiveState get state => delegate.state ?? NERoomItemLiveState.invalid;

  /// 是否开启聊天室
  @override
  set liveChatRoomEnable(bool liveChatRoomEnable) =>
      delegate.liveChatRoomEnable = liveChatRoomEnable;

  @override
  bool get liveChatRoomEnable => delegate.liveChatRoomEnable;

  /// 直播布局
  @override
  set liveLayout(NELiveLayout liveLayout) =>
      delegate.liveLayout = liveLayout.index;

  @override
  NELiveLayout get liveLayout => NELiveLayout.values.firstWhere(
        (element) => element.index == delegate.liveLayout,
    orElse: () => NELiveLayout.none,
  );

  /// 直播聊天室id
  @override
  String? get chatRoomId => delegate.chatRoomId;

  /// hls 拉流地址
  @override
  String? get hlsPullUrl => delegate.hlsPullUrl;

  /// http 拉流地址
  @override
  String? get httpPullUrl => delegate.httpPullUrl;

  /// rtmp 拉流地址
  @override
  String? get rtmpPullUrl => delegate.rtmpPullUrl;

  /// 直播地址
  @override
  String? get liveUrl => delegate.liveUrl;

  @override
  NELiveAuthLevel get webAccessControlLevel =>
      LiveAuthLevelExtension.get(delegate.liveWebAccessControlLevel);

  @override
  set webAccessControlLevel(NELiveAuthLevel level) =>
    delegate.liveWebAccessControlLevel = level.index;

  @override
  set userList(List<String>? userList) {
    _userList = userList;
    if (roomData != null) {
      delegate.liveAVRoomUids = userList
          ?.where((userId) => roomData!.userId2User.containsKey(userId))
          .map((userId) => roomData!.userId2User[userId]!.rtcId)
          .toList(growable: false);
    }
  }

  @override
  List<String>? get userList {
    if (roomData != null) {
      _userList = delegate.liveAVRoomUids
          ?.where((rtcId) => roomData!.rtcId2User.containsKey(rtcId))
          .map((rtcId) => roomData!.rtcId2User[rtcId]!.userId)
          .toList();
    }
    return _userList;
  }

  _InRoomLiveInfo copy() {
    return _InRoomLiveInfo(roomData, delegate.copy());
  }

  @override
  bool operator ==(Object other) =>
      identical(this, other) ||
      other is _InRoomLiveInfo &&
          runtimeType == other.runtimeType &&
          delegate == other.delegate;

  @override
  int get hashCode => delegate.hashCode;
}