// Copyright (c) 2014-2020 NetEase, Inc.
// All right reserved.
part of room_kit;

/// 预定会议实现类
class _NEPreRoomServiceImpl extends NEPreRoomService {
  static const _tag = 'NEPreRoomService';

  static final _NEPreRoomServiceImpl _instance =
      _NEPreRoomServiceImpl._();

  factory _NEPreRoomServiceImpl() => _instance;

  final List<ScheduledRoomStatusListener<List<NERoomItem>>> _listeners =
      <ScheduledRoomStatusListener<List<NERoomItem>>>[];
  late final _NEPreRoomBeautyControllerImpl beautyController;

  _NEPreRoomServiceImpl._() {
    beautyController =  _NEPreRoomBeautyControllerImpl();

    // 每次回调是增量更新
    InRoomRepository.onServerControlReceived.listen((MeetingAction action) {
      if (action is ScheduledRoomStatusChangeAction) {
        for (var callback in _listeners) {
          callback(action.list, true);
        }
      }
    });

    // 每次重新登录，重新拉取一次全量列表
    AuthRepository.imOnlineStatusChanged.listen((event) {
      if (event == StatusCode.logined && _listeners.isNotEmpty) {
        getRoomList([
          NERoomItemStatus.init,
          NERoomItemStatus.started,
          NERoomItemStatus.ended,
          NERoomItemStatus.cancel,
          NERoomItemStatus.recycled
        ]).then((result) {
          if (result.code == RoomErrorCode.success &&
              (result.data is List<NERoomItem>)) {
            for (var callback in _listeners) {
              callback(result.nonNullData, false);
            }
          }
        });
      }
    });
  }

  @override
  Future<NEResult<NERoomItem>> scheduleRoom(NERoomItem item) {
    Alog.i(
        tag: _tag,
        moduleName: _moduleName,
        type: AlogType.api,
        content: 'scheduleRoom ,item=${item.toString()}');
    // if (item.subject == null) {
    //   return Future.value(NEResult(code: ErrorCode.paramsError));
    // }
    return PreRoomRepository.scheduleRoom(item);
  }

  @override
  Future<NEResult<NERoomItem>> editRoom(NERoomItem item) {
    Alog.i(
        tag: _tag,
        moduleName: _moduleName,
        type: AlogType.api,
        content: 'editRoom ,item=${item.toString()}');
    // if (item.subject == null) {
    //   return Future.value(NEResult(code: ErrorCode.paramsError));
    // }
    return PreRoomRepository.editRoom(item);
  }

  @override
  Future<NEResult<void>> cancelRoom(int roomUniqueId) {
    Alog.i(
        tag: _tag,
        moduleName: _moduleName,
        type: AlogType.api,
        content: 'cancelRoom $roomUniqueId');
    return PreRoomRepository.cancelRoom(roomUniqueId);
  }

  @override
  Future<NEResult<NERoomItem>> getRoomItemByUniqueId(int roomUniqueId) {
    Alog.i(
        tag: _tag,
        moduleName: _moduleName,
        type: AlogType.api,
        content: 'getRoomItemById $roomUniqueId');
    return PreRoomRepository.getRoomItemById(roomUniqueId);
  }

  @override
  Future<NEResult<List<NERoomItem>>> getRoomList(
      List<NERoomItemStatus> status) {
    Alog.i(
        tag: _tag,
        moduleName: _moduleName,
        type: AlogType.api,
        content: 'getRoomList ,status${status.toString()}');
    return PreRoomRepository.getRoomList(status);
  }

  @override
  void addScheduledRoomStatusListener(
      ScheduledRoomStatusListener<List<NERoomItem>> callback) {
    Alog.i(
        tag: _tag,
        moduleName: _moduleName,
        type: AlogType.api,
        content: 'addScheduledRoomStatusListener');
    if (_listeners.contains(callback)) {
      return;
    }
    _listeners.add(callback);
  }

  @override
  void removeScheduledRoomStatusListener(
      ScheduledRoomStatusListener<List<NERoomItem>> callback) {
    Alog.i(
        tag: _tag,
        moduleName: _moduleName,
        type: AlogType.api,
        content: 'removeScheduledRoomStatusListener');
    if (_listeners.contains(callback)) {
      _listeners.remove(callback);
    }
  }

  @override
  NEPreRoomBeautyController getPreRoomBeautyController() {
    return beautyController;
  }

}
