// Copyright (c) 2014-2020 NetEase, Inc.
// All right reserved.
part of room_kit;

class _NERoomSDKImpl extends NERoomKit with EventTrackMixin, _RoomAloggerMixin {
  static const _tag = 'NERoomSDKImpl';
  NERoomKitConfig? _lastConfig;
  final _NERoomServiceImpl roomService = _NERoomServiceImpl();
  NEAuthService authService = _NEAccountServiceImpl();
  NESettingsService settingsService = _NESettingsServiceImpl();
  NEPreRoomService preRoomService = _NEPreRoomServiceImpl();
  NEHttpApiService httpApiService = _NEHttpApiServiceImpl();

  Map? customServerConfig;

  _NERoomSDKImpl();

  @override
  NERoomKitConfig? get config => _lastConfig;

  @override
  Future<NEResult<void>> initialize(NERoomKitConfig config) async {
    Alog.i(
      tag: _tag,
      type: AlogType.api,
      moduleName: _moduleName,
      content: 'config -> $config',
    );
    trackPeriodicEvent(TrackEventName.sdkInit);
    var result = NEResult(code: RoomErrorCode.success);
    if (config == _lastConfig) {
      return Future.value(result);
    }
    await initServerConfig(config);
    var alogInit = await NERoomLogService().init(loggerConfig: config.aLoggerConfig);
    if (!alogInit) {
      return NEResult(
          code: RoomErrorCode.failed,
          msg: NERoomSDKStrings.roomLogPathParamsError);
    }
    await _subscriptionGlobalStatus();
    var isInit = await ServiceRepository.initialize(
        config.appKey, null, config.reuseNIM, NERoomLogService()._nimSDKPath);
    if (isInit) {
      _lastConfig = config;
      result = NEResult(code: RoomErrorCode.success);

      /// 获取基础配置成功后，返回初始化结果
      await SDKConfig.initialize();
    } else {
      result = NEResult(code: RoomErrorCode.failed);
    }
    return Future.value(result);
  }

  Future<bool> initServerConfig(NERoomKitConfig config) async {
    if (customServerConfig == null) {
      final serverConfig = config.serverConfig;
      if (serverConfig != null && serverConfig.isNotEmpty) {
        assert(() {
          print('custom server config: $serverConfig');
          return true;
        }());
        try {
          final decodedCustomServerConfig = json.decode(serverConfig);
          if (decodedCustomServerConfig is Map) {
            final serverUrl = decodedCustomServerConfig['serverUrl'] as String?;
            if (serverUrl != null) {
              ServersConfig().serverUrl = serverUrl;
              customServerConfig = decodedCustomServerConfig;
              return true;
            }
          }
        } catch (e) {
          commonLogger.e('parse asset server config error:\n$e');
        }
      } else {
        final hideServerUrl = config.extras?['serverUrl'] as String?;
        if (hideServerUrl != null && hideServerUrl.startsWith(r'http')) {
          ServersConfig().serverUrl = hideServerUrl;
        }
      }
    }
    return false;
  }

  StreamSubscription? _onlineSubscription;
  Future<void> _subscriptionGlobalStatus() async {
    await _onlineSubscription?.cancel();
    _onlineSubscription =
        AuthRepository.onlineStatusChanged.listen((status) async {
      if (status == AuthState.authorized) {
        //登录完成后尝试获取配置信息，配置请求如果不成功，网络成功的时候再请求一次
        if (!SDKConfig.initialized) {
          commonLogger.i('network recovery, fetch global config again');
          await SDKConfig.initialize();
        }
      }
    });
  }

  @override
  NERoomService getRoomService() => roomService;

  @override
  NEInRoomService? getInRoomService() => roomService.currentInRoomService;

  @override
  NEAuthService getAuthService() => authService;

  @override
  NESettingsService getSettingsService() => settingsService;

  @override
  NEPreRoomService getPreRoomService() => preRoomService;

  @override
  NEHttpApiService getHttpApiService() {
    return httpApiService;
  }
}
