// Copyright (c) 2014-2020 NetEase, Inc.
// All right reserved.
part of room_kit;

abstract class NEInRoomService {
  /// 在房间里获取音频控制器实例.
  NEInRoomAudioController getInRoomAudioController();

  /// 在房间里获取云端录制控制器实例.
  NEInRoomCloudRecordController getInRoomCloudRecordController();

  /// 在房间里获取聊天室控制器实例.
  NEInRoomChatController getInRoomChatController();

  /// 在房间里获取美颜控制器实例.
  NEInRoomBeautyController getInRoomBeautyController();

  /// 在房间里获取白板控制器实例.
  NEInRoomWhiteboardController getInRoomWhiteboardController();

  /// 获取房间直播控制器
  NEInRoomLiveStreamController getInRoomLiveStreamController();

  /// 获取房间屏幕共享控制器
  NEInRoomScreenShareController getInRoomScreenShareController();

  /// 获取房间用户控制器
  NEInRoomUserController getInRoomUserController();

  /// 获取房间视频控制器
  NEInRoomVideoController getInRoomVideoController();

  /// 添加房间监听
  void addListener(NEInRoomServiceListener listener);

  /// 移除房间监听
  void removeListener(NEInRoomServiceListener listener);

  /// 添加房间状态监听实例，用于接收房间状态变更通知
  ///
  /// * [observer] 要添加的监听实例
  void addRoomLifeCycleObserver(NERoomLifeCycleObserver observer);

  /// 移除对应的房间状态的监听实例
  ///
  /// * [observer] 要移除的监听实例
  void removeRoomLifeCycleObserver(NERoomLifeCycleObserver observer);

  /// 开始连接房间
  void startConnect();

  /// 房间是否锁定
  bool isRoomLocked();

  /// 主持人锁定房间
  Future<NEResult<void>> lockRoom(bool isLock);

  /// 获取当前房间用户数
  int getUserCount();

  /// 获取当前房间内的用户列表
  List<NEInRoomUserInfo> getAllUsers();

  /// 通过userId查询用户信息
  NEInRoomUserInfo? getUserInfoById(String userId);

  /// 获取我的用户id
  String getMyUserId();

  bool isMyself(String userId);

  /// 获取我的用户信息
  NEInRoomUserInfo? getMyUserInfo();

  /// 是否为主持人
  bool isHostUser(String userId);

  /// 自己是否为主持人
  bool isMySelfHost();

  String? getHostUserId();

  /// 是否允许自己解除静音
  bool isParticipantsUnmuteSelfAllowed();

  /// 离开当前房间
  /// [closeIfHost] 退出是否同步关闭房间
  /// 不论请求关闭房间的结果是成功还是失败，调用该接口当前用户都会从当前房间退出
  Future<NEResult<void>> leaveCurrentRoom(bool closeIfHost);

  /// 获取当前房间Id
  String getCurrentRoomId();

  /// 获取当前房间信息
  NERoomInfo? getCurrentRoomInfo();

  /// 添加数据统计监听
  void addRtcStatsEventListener(NERtcStatsEventListener listener);

  /// 移除数据监听
  void removeRtcStatsEventListener(NERtcStatsEventListener listener);

  /// 添加设备监听
  void addRtcDeviceEventListener(NERtcDeviceEventCallback listener);

  /// 移除设备监听
  void removeRtcDeviceEventListener(NERtcDeviceEventCallback listener);
}

/// 当前通话统计回调
abstract class NERtcStatsEventListener {
  /// 当前通话统计回调
  ///  SDK 定期向 App 报告当前通话的统计信息，每 2 秒触发一次。
  void onRtcStats(NERtcStats stats);

  /// 本地音频流统计信息回调
  ///  该回调描述本地设备发送音频流的统计信息，每 2 秒触发一次。
  void onLocalAudioStats(NERtcAudioSendStats stats);

  /// 通话中远端音频流的统计信息回调。
  ///  该回调描述远端用户在通话中端到端的音频流统计信息，每 2 秒触发一次。
  void onRemoteAudioStats(List<NERoomAudioRecvStats> statsList);

  ///  本地视频流统计信息回调
  ///  该回调描述本地设备发送视频流的统计信息，每 2 秒触发一次。
  void onLocalVideoStats(NERtcVideoSendStats stats);

  ///  通话中远端视频流的统计信息回调
  ///  该回调描述远端用户在通话中端到端的视频流统计信息，每 2 秒触发一次。
  void onRemoteVideoStats(List<NERoomVideoRecvStats> statsList);

  ///  通话中每个用户的网络上下行质量报告回调
  ///  该回调描述每个用户在通话中的网络状态，每 2 秒触发一次。
  void onNetworkQuality(List<NERoomNetworkQualityInfo> statsList);
}
