// Copyright (c) 2014-2021 NetEase, Inc.
// All right reserved.
part of room_kit;

abstract class NEInRoomUserInfo {

  /// 房间用户id
  String get userId;

  /// 房间用户昵称
  String get displayName;

  /// 当前举手状态
  NERaiseHandDetail get raiseHandDetail;

  /// 是否正在白板共享
  bool get isWhiteBoardSharing;

  /// 是否正在屏幕共享
  bool get isScreenSharing;

  /// 是否为焦点视频
  bool get isPinned;

  /// 客户端类型
  int get clientType;

  /// 角色身份，1成员，2主持人，3管理员，4隐藏
  int get roleType;

  /// 成员标签，自定义，最大长度50
  String? get tag;

  /// 视频状态
  NERoomVideoStatus get videoStatus;

  /// 音频状态
  NERoomAudioStatus get audioStatus;

  /// 白板相互协作权限
  NERoomWhiteBoardInteractionStatus get whiteBoardInteractStatus;
}

abstract class NERaiseHandDetail {

  /// [NEHandsUpStatus]
  int get status;

  int get timestamp;

  bool get isRaisingHand => status == NEHandsUpStatus.up;
}

/// [AVState]
enum NERoomAudioStatus {
  on, /// 开启
  off, /// 关闭
  mutedByHost, /// 被主持人关闭
  waitingOpen, /// 主持人请求打开
}

NERoomAudioStatus indexToAudioStatus(int index) =>
    NERoomAudioStatus.values.firstWhere(
      (element) => element.index + 1== index,
      orElse: () => NERoomAudioStatus.off,
    );

enum NERoomVideoStatus {
  on, /// 开启
  off, /// 关闭
  mutedByHost, /// 被主持人关闭
  waitingOpen, /// 主持人请求打开
}

NERoomVideoStatus indexToVideoStatus(int index) =>
    NERoomVideoStatus.values.firstWhere(
      (element) => element.index + 1 == index,
      orElse: () => NERoomVideoStatus.off,
    );

enum NERoomScreenShareStatus {
  start, /// 共享开始
  end,   /// 共享结束
  stopByHost, /// 共享被终止
}

enum NERoomWhiteBoardShareStatus {
  start, /// 共享开始
  end,   /// 共享结束
  stopByHost, /// 共享被终止
}

/// [InteractionStatus]
enum NERoomWhiteBoardInteractionStatus {
  close, /// 关闭
  open, /// 开启
}

extension _WhiteBoardInteractionStatusExtension
    on NERoomWhiteBoardInteractionStatus {
  static NERoomWhiteBoardInteractionStatus get(int mode) {
    switch (mode) {
      case InteractionStatus.close:
        return NERoomWhiteBoardInteractionStatus.close;
      case InteractionStatus.open:
        return NERoomWhiteBoardInteractionStatus.open;
      default:
        return NERoomWhiteBoardInteractionStatus.close;
    }
  }
}