// Copyright (c) 2014-2020 NetEase, Inc.
// All right reserved.
part of room_kit;

typedef ScheduledRoomStatusListener<T> = void Function(T data, bool incremental);

/// 房间前服务， 主要提供房间开始前的一系列方法， eg： 房间预定
abstract class NEPreRoomService {

  /// 预约房间
  Future<NEResult<NERoomItem>> scheduleRoom(NERoomItem item);

  /// 编辑预约房间
  Future<NEResult<NERoomItem>> editRoom(NERoomItem item);

  /// 取消预约房间，开始前可以取消
  Future<NEResult<void>> cancelRoom(int roomUniqueId);

  /// 根据唯一id获取房间信息
  Future<NEResult<NERoomItem>> getRoomItemByUniqueId(int roomUniqueId);

  /// 根据房间状态查询房间信息列表， 不传默认返回NEMeetingItemStatus.init, NEMeetingItemStatus.started
  Future<NEResult<List<NERoomItem>>> getRoomList(List<NERoomItemStatus> status);

  /// 注册监听预定房间状态变更回调
  void addScheduledRoomStatusListener(ScheduledRoomStatusListener<List<NERoomItem>> listener);

  /// 反注册监听预定房间状态变更回调
  void removeScheduledRoomStatusListener(ScheduledRoomStatusListener<List<NERoomItem>> listener);

  /// 美颜控制器实例.
  NEPreRoomBeautyController getPreRoomBeautyController();
}
