// Copyright (c) 2014-2020 NetEase, Inc.
// All right reserved.

part of room_kit;

/// 提供房间SDK初始化时必要的参数和配置信息
class NERoomKitConfig {
  /// 房间App Key，不能为空
  final String appKey;

  /// 是否复用IM通道
  final bool reuseNIM;

  final String? serverConfig;

  /// 日志参数配置
  final ALoggerConfig aLoggerConfig;

  final Map<String,dynamic>? extras;

  NERoomKitConfig({
    required this.appKey,
    this.serverConfig,
    this.reuseNIM = false,
    this.extras,
    this.aLoggerConfig = const ALoggerConfig(),
  });

  @override
  bool operator ==(other) {
    if (other is! NERoomKitConfig) {
      return false;
    }
    return appKey == other.appKey &&
        reuseNIM == other.reuseNIM &&
        serverConfig == other.serverConfig &&
        aLoggerConfig == other.aLoggerConfig &&
        mapEquals(extras, other.extras)
    ;
  }

  @override
  int get hashCode => hashValues(
        appKey,
        reuseNIM,
        serverConfig,
        aLoggerConfig,
        extras,
      );
}

/// 房间SDK全局接口，提供初始化、管理其他房间相关子服务的能力
abstract class NERoomKit {
  static final NERoomKit _instance = _NERoomSDKImpl();

  /// 获取房间SDK实例
  static NERoomKit get instance => _instance;

  NERoomKitConfig? get config;

  /// 初始化SDK
  ///
  /// * [config]   初始化配置对象
  Future<NEResult<void>> initialize(NERoomKitConfig config);

  /// 获取用于创建或加入房间的房间服务。
  NERoomService getRoomService();

  /// 获取用于查询账号信息的账号服务
  NEAuthService getAuthService();

  /// 获取房间设置服务
  NESettingsService getSettingsService();

  /// 获取房间前服务
  NEPreRoomService getPreRoomService();

  /// 获取房间内服务
  NEInRoomService? getInRoomService();

  /// 房间网络服务
  NEHttpApiService getHttpApiService();
}

/// 房间登陆状态回调
abstract class NERoomAuthListener {
  /// 被踢出
  void onKickOut();

  /// 账号信息过期通知，原因为用户修改了密码，应用层随后应该重新登录
  void onAuthInfoExpired();
}
