// Copyright (c) 2014-2020 NetEase, Inc.
// All right reserved.
part of room_kit;

class NERoomLifeCycleObserverOwner {
  static const _tag = 'NERoomLifeCycleObserverOwner';

  final _roomLifeCycleObserverSet = <NERoomLifeCycleObserver>{};

  NERoomLifeCycleObserverOwner();

  static const idle = -1;
  static const connecting = 0;
  static const connectFail = 1;
  static const connected = 2;
  static const disconnecting = 3;
  static const disconnected = 4;

  var _currentState = idle;
  int? _stateParameter;

  void addObserver(NERoomLifeCycleObserver observer) {
    _roomLifeCycleObserverSet.add(observer);
  }

  void removeObserver(NERoomLifeCycleObserver observer) {
    _roomLifeCycleObserverSet.remove(observer);
  }

  void notifyRoomOnConnecting() {
    if (_currentState >= connecting) return;
    Alog.i(
        tag: _tag,
        moduleName: _moduleName,
        content: 'notifyRoomOnConnecting');
    _currentState = connecting;
    _roomLifeCycleObserverSet.toList().forEach((element) {
        element.onConnecting();
    });
  }

  void notifyRoomConnectFail(int reason) {
    if (_currentState >= connectFail) return;
    Alog.i(
        tag: _tag,
        moduleName: _moduleName,
        content: 'notifyRoomConnectFail: $reason');
    _currentState = connectFail;
    _stateParameter = reason;
    _roomLifeCycleObserverSet.toList().forEach((element) {
      element.onConnectFail(reason);
    });
  }

  void notifyRoomOnConnected() {
    if (_currentState >= connected) return;
    Alog.i(
        tag: _tag,
        moduleName: _moduleName,
        content: 'notifyRoomOnConnected');
    _currentState = connected;
    _roomLifeCycleObserverSet.toList().forEach((element) {
        element.onConnected();
    });
  }

  void notifyRoomOnDisConnecting() {
    if (_currentState >= disconnecting) return;
    Alog.i(
        tag: _tag,
        moduleName: _moduleName,
        content: 'notifyRoomOnDisConnecting');
    _currentState = disconnecting;
    _roomLifeCycleObserverSet.toList().forEach((element) {
        element.onDisconnecting();
    });
  }

  void notifyRoomOnDisConnected(int reason) {
    if (_currentState >= disconnected) return;
    Alog.i(
        tag: _tag,
        moduleName: _moduleName,
        content: 'notifyRoomOnDisConnected: $reason');
    _currentState = disconnected;
    _stateParameter = reason;
    _roomLifeCycleObserverSet.toList().forEach((element) {
        element.onDisconnected(reason);
    });
  }

  void notifyRoomOnIdle() {
    if (_currentState != connectFail || _currentState != disconnected) return;
    Alog.i(tag: _tag, moduleName: _moduleName, content: 'notifyRoomOnIdle');
    _currentState = idle;
    _roomLifeCycleObserverSet.toList().forEach((element) {
      element.onIdle();
    });
  }

  void clear() {
    _roomLifeCycleObserverSet.clear();
  }
}

Future<NEResult<void>> onDeny() {
  return Future.sync(() => NEResult(
        code: RoomErrorCode.failed,
        msg: NERoomSDKStrings.apiCallTooFrequent,
      ));
}

Future<NEResult<void>> onRoomIsUnderGoing() {
  return Future.sync(() => NEResult(
        code: RoomErrorCode.failed,
        msg: NERoomSDKStrings.roomIsUnderGoing,
      ));
}

Future<NEResult<JoinRoomInfo>> onJoinRoomFail() {
  return Future.sync(() => NEResult(
        code: RoomErrorCode.failed,
        msg: NERoomSDKStrings.apiCallTooFrequent,
      ));
}

mixin _RoomAloggerMixin {

  Alogger? _apiLogger, _commonLogger;

  Alogger get apiLogger {
    _apiLogger ??= Alogger.api(runtimeType.toString(), _moduleName);
    return _apiLogger!;
  }

  Alogger get commonLogger {
    _commonLogger ??= Alogger.normal(runtimeType.toString(), _moduleName);
    return _commonLogger!;
  }

}
