// Copyright (c) 2014-2020 NetEase, Inc.
// All right reserved.

part of room_service;

class ChatRoomApiHelper {
  ChatRoomApiHelper._();

  static Future<NEResult<T>> execute<T>(BaseApi api, {bool isCheckIM = false}) {
    var completer = Completer<NEResult<T>>();
    if (isCheckIM) {
      _AuthManager()._imLoginCheck().then((code) {
        if (code == RoomErrorCode.success) {
          completer.complete(_ApiHelper._execute(api));
        } else {
          completer
              .complete(Future.value(NEResult(code: RoomErrorCode.notLogin)));
        }
      });
    } else {
      completer.complete(_ApiHelper._execute(api));
    }

    return completer.future;
  }

  static Future<NEResult<void>> _sendChatRoomMessage(ChatRoomMessage message) {
    if (_AuthManager().imNetBroken()) {
      return Future.value(NEResult(code: RoomErrorCode.networkError));
    }
    return execute(ChatRoomSendMessageApi(message));
  }

  /// 进入聊天室
  static Future<NEResult<EnterChatRoomResultData>> _enterChatRoom(
      EnterChatRoomData data, int retryCount,
      {RoomLinkAddress? linkAddress}) {
    return execute(
        ChatRoomEnterApi(data, retryCount, linkAddress: linkAddress));
  }

  /// exit chat ro
  static Future<NEResult<void>> _exitChatRoom(String roomId) {
    return execute(ChatRoomExitApi(roomId));
  }

  /// chat room message receive
  static Stream<List<ChatRoomMessage>> get _chatRoomReceiveMessage =>
      _ChatRoomReceiveMessageApi().listen();

  static Future<NEResult<void>> _updateChatRoomRole(
          String roomId, ChatRoomMemberUpdate memberUpdate) =>
      execute(ChatRoomUpdateRoleApi(roomId, memberUpdate));
}
