// Copyright (c) 2014-2020 NetEase, Inc.
// All right reserved.

part of room_service;

class HttpApiHelper {
  HttpApiHelper._();

  static Future<NEResult<T>> execute<T>(BaseApi api, {bool isCheckIM = false}) {
    var completer = Completer<NEResult<T>>();
    if (isCheckIM) {
      _AuthManager()._imLoginCheck().then((code) {
        if (code == RoomErrorCode.success) {
          completer.complete(_ApiHelper._execute(api));
        } else {
          completer
              .complete(Future.value(NEResult(code: RoomErrorCode.notLogin)));
        }
      });
    } else {
      completer.complete(_ApiHelper._execute(api));
    }

    return completer.future;
  }

  static Future<NEResult<dynamic>> sendRequest(NEHttpApiRequest request) {
    return execute(_GenericHttpApiRequest(request));
  }

  /// 网易会议登录(token登陆)
  static Future<NEResult<_LoginResponse>> _loginWithToken(
          String accountId, String accountToken) =>
      execute(
          _LoginWithTokenApi(_LoginWithTokenRequest(accountId, accountToken)));

  /// 网易会议登录(密码登陆)
  static Future<NEResult<_LoginResponse>> _loginWithNEMeeting(
          String username, String password) =>
      execute(_LoginWithNEMeetingApi(_LoginByPwdRequest(username, password)));

  /// 网易会议登录(SSO登陆)
  static Future<NEResult<_LoginResponse>> _loginWithSSOToken(String ssoToken) =>
      execute(_LoginWithSSOTokenApi(_LoginWithSSOTokenRequest(ssoToken)));

  /// 创建会议
  static Future<NEResult<JoinRoomInfo>> _createMeeting(
          String nickName, String? meetingId, String? password,
      String? tag, int video, int audio, bool enableChatroom, bool cloudRecordOn, Map? scene) =>
      execute(
          _CreateMeetingApi(_CreateMeetingRequest(
              nickName: nickName,
              meetingId: meetingId,
              password: password,
              tag:tag,
              video: video,
              audio: audio,
              enableChatroom: enableChatroom,
              cloudRecordOn:cloudRecordOn,
              scene:scene)),
          isCheckIM: true);

  /// 加入会议
  static Future<NEResult<JoinRoomInfo>> _joinMeeting(String nickName,
          String meetingId,String? tag, int video, int audio, String? password) =>
      execute(
          _JoinMeetingApi(_JoinMeetingRequest(
              nickName: nickName,
              meetingId: meetingId,
              tag: tag,
              video: video,
              audio: audio,
              password: password)),
          isCheckIM: true);

  /// 匿名加入会议信息
  static Future<NEResult<AnonymousJoinMeetingInfo>> _anonymousJoinMeeting(
          String meetingId,
          String nickName,
          String? tag,
          int video,
          int audio,
          String? password) =>
      execute(_AnonymousJoinMeetingApi(_AnonymousJoinMeetingRequest(
          meetingId: meetingId,
          nickName: nickName,
          tag: tag,
          video: video,
          audio: audio,
          password: password,
          appKey: _AuthManager().appKey,
          domain: _AuthManager().domain)));

  /// 获取全局配置
  static Future<NEResult<_SDKGlobalConfig>> _getSDKGlobalConfig(
      String bundleId) {
    return execute(_GetConfigApi(bundleId));
  }

  /// 获取聊天室link地址
  static Future<NEResult<List<String>>> _fetchRoomLink(String roomId) {
    return execute(_FetchRoomLinkApi(roomId));
  }

  /// 预约会议
  static Future<NEResult<NERoomItem>> _scheduleRoom(NERoomItem item) {
    return execute(_ScheduleMeetingApi(item));
  }

  /// 编辑预约会议
  static Future<NEResult<NERoomItem>> _editRoom(NERoomItem item) {
    return execute(_EditMeetingApi(item));
  }

  ///取消预约会议，开始前可以取消
  static Future<NEResult<void>> _cancelRoom(int meetingUniqueId) {
    return execute(_CancelMeetingApi(meetingUniqueId));
  }

  ///删除预约会议
  static Future<NEResult<void>> _deleteRoom(int meetingUniqueId) {
    return execute(_DeleteMeetingApi(meetingUniqueId));
  }

  ///根据唯一id获取会议信息
  static Future<NEResult<NERoomItem>> _getRoomItemById(
      int meetingUniqueId) {
    return execute(_GetMeetingItemByIdApi(meetingUniqueId));
  }

  ///根据会议状态查询会议信息列表
  static Future<NEResult<List<NERoomItem>>> _getRoomList(
      List<NERoomItemStatus> status) {
    return execute(_GetMeetingListByStatusApi(status));
  }

  ///保存用户信息配置
  static Future<NEResult<void>> _saveSettingsApi(BeautySettings obj) {
    return execute(_SaveSettingsApi(obj));
  }

  ///获取用户信息配置
  static Future<NEResult<AccountSettings>> _getSettingsApi() {
    return execute(_GetSettingsApi());
  }

  ///获取用户美颜证书
  static Future<NEResult<Uint8List>> _getBeautyLicenseApi(
      String licenseUrl, ProgressCallback progressCallback) {
    return execute(_DownloadBeautyLicenseApi(licenseUrl, progressCallback));
  }
}
