// Copyright (c) 2014-2020 NetEase, Inc.
// All right reserved.

part of room_service;

class _Executors {
  Map<String, dynamic> get _baseHeaders => {
    'accountToken': _AuthManager().accountToken,
    'accountId': _AuthManager().accountId,
    'clientType': DeviceInfo.clientType,
    'meetingSdkVersion': SDKConfig.sdkVersion,
    'appKey': _AuthManager().appKey,
    'deviceId': DeviceInfo.deviceId,
  };

  static _Executors? _instance;

  final _NIMExecutor _nimExecutor = _NIMExecutor();
  final _HttpExecutor _httpExecutor = _HttpExecutor();

  factory _Executors() {
    return _instance ??= _Executors._internal();
  }

  _Executors._internal();

  Future<NEResult<PassthroughProxyData>> executeOverNIM(String path, Map<String, dynamic>? headers, Map data) {
    return _nimExecutor._execute(path, mergeHeaders(_baseHeaders, headers), data);
  }

  Future<http.Response?> executeOverHttp(String path, Map<String, dynamic>? headers, data) {
    return _httpExecutor._execute(path, mergeHeaders(_baseHeaders, headers), data);
  }

  static Map<String, dynamic>? mergeHeaders(Map<String, dynamic>? lhs, Map<String, dynamic>? rhs) {
    if (lhs != null || rhs != null) {
      return {
        if (lhs != null) ...(lhs..removeWhere((key, value) => value == null)),
        if (rhs != null) ...(rhs..removeWhere((key, value) => value == null)),
      };
    }
    return null;
  }
}
