// Copyright (c) 2014-2020 NetEase, Inc.
// All right reserved.

part of room_service;

const _tag = '_HttpExecutor';

class _HttpExecutor {
  static _HttpExecutor? _instance;

  factory _HttpExecutor() {
    return _instance ??= _HttpExecutor._internal();
  }

  late http.Dio dio;
  final ServersConfig _serversConfig = ServersConfig();

  _HttpExecutor._internal() {
    var options = http.BaseOptions(
        baseUrl: _serversConfig.baseUrl,
        connectTimeout: _serversConfig.connectTimeout,
        receiveTimeout: _serversConfig.receiveTimeout);

    dio = http.Dio(options);
  }

  Future<http.Response?> _execute(
      String path, Map<String, dynamic>? headers, data) async {
    http.Response? response;
    try {
      var options = http.Options();
      options.headers = headers;
      Alog.i(
          tag: _tag,
          moduleName: _moduleName,
          content:
              'MeetingSDK, HttpExecutor: req path=$path params=${data?.toString()}');
      if (path.startsWith('http')) {
        response =
            await dio.postUri(Uri.parse(path), data: data, options: options);
      } else {
        response = await dio.post(path, data: data, options: options);
      }
    } on http.DioError catch (e) {
      Alog.e(
          tag: _tag,
          moduleName: _moduleName,
          content: 'MeetingSDK, HttpExecutor: res path=$path error=$e');
    }
    return response;
  }

  Future<http.Response?> download(
      String url, ProgressCallback onReceiveProgress) async {
    http.Response? response;
    var options = http.Options(responseType: ResponseType.bytes);
    try {
      Alog.d(
          tag: _tag,
          moduleName: _moduleName,
          content: 'MeetingSDK, HttpExecutor:  download file $url');
      response = await dio.get(url,
          onReceiveProgress: onReceiveProgress, options: options);
    } on DioError catch (e) {
      Alog.d(
          tag: _tag,
          moduleName: _moduleName,
          content: 'MeetingSDK, HttpExecutor:  download file $url error=$e');
    }
    return response;
  }
}
