// Copyright (c) 2014-2020 NetEase, Inc.
// All right reserved.

part of room_service;

class _NIMExecutor {
  static _NIMExecutor? _instance;

  factory _NIMExecutor() => _instance ??= _NIMExecutor._internal();

  late PassthroughService service;

  late PassthroughObserver observer;

  _NIMExecutor._internal() {
    var plugin = NimPlugin();
    service = plugin.getPassthroughService();
    observer = plugin.getPassthroughObserver();
  }

  String _flatHeader(Map<String, dynamic>? header) {
    header ??= <String, dynamic>{};
    return json.encode(header);
  }

  String? _flatBody(Map data) {
    if (data.isEmpty) {
      return null;
    }
    return json.encode(data);
  }

  Future<NEResult<PassthroughProxyData>> _execute(
      String path, Map<String, dynamic>? headers, Map data) {
    Alog.i(
        tag: _tag,
        moduleName: _moduleName,
        content: 'NIMExecutor: req path=$path params=${data.toString()}');
    var data2 = PassthroughProxyData(
        path: path, header: _flatHeader(headers), body: _flatBody(data));
    final completer = Completer<NEResult<PassthroughProxyData>>();
    service.httpProxy(data2, (int code, PassthroughProxyData? data) {
      Alog.i(
          tag: _tag,
          moduleName: _moduleName,
          content:
              'NIMExecutor: res path=${data?.path ?? path} code=$code body=${data?.body}');
      completer.complete(NEResult(code: code, data: data));
    });
    return completer.future;
  }
}
