// Copyright (c) 2014-2020 NetEase, Inc.
// All right reserved.
part of room_service;

class _SDKPreferences {
  static const String _keyDeviceId = 'meeting_sdk_deviceId';
  static const String _keyAccountId = 'meeting_sdk_account_id';
  static const String _keyAccountToken = 'meeting_sdk_account_token';
  static const String _keyAppKey = 'meeting_sdk_app_key';

  static Future<void> setDeviceId(String deviceId) async {
    await _setString(_keyDeviceId, deviceId);
  }

  static Future<String?> getDeviceId() {
    return _getString(_keyDeviceId, '');
  }

  static Future<void> setAccountId(String? accountId) async {
    await _setString(_keyAccountId, accountId);
  }

  static Future<String?> get accountId {
    return _getString(_keyAccountId);
  }

  static Future<void> setAccountToken(String? accountToken) async {
    await _setString(_keyAccountToken, accountToken);
  }

  static Future<String?> get accountToken {
    return _getString(_keyAccountToken);
  }

  static Future<void> setAppKey(String? appKey) async {
    await _setString(_keyAppKey, appKey);
  }

  static Future<String?> get appKey {
    return _getString(_keyAppKey);
  }

  static SharedPreferences? _prefs;

  static Future<SharedPreferences> _ensureSharedPreferences() async {
    _prefs ??= await SharedPreferences.getInstance();
    return _prefs!;
  }

  static Future<bool> _setString(String key, String? value) async {
    await _ensureSharedPreferences();
    if (value == null) {
      return await _prefs!.remove(key);
    } else {
      return await _prefs!.setString(key, value);
    }
  }

  static Future<String?> _getString(String key, [String? fallback]) async {
    await _ensureSharedPreferences();
    return _prefs!.getString(key) ?? fallback;
  }
}
