// Copyright (c) 2014-2020 NetEase, Inc.
// All right reserved.

part of room_service;

class JoinRoomInfo {
  ///	随机会议码,9位数字；个人会议码，10位数字
  final String meetingId;

  /// 公有云音视频房间名称
  final String avRoomCName;

  ///公有云音视频房间id
  final String avRoomCid;

  ///公有云音视频房间成员uid
  final int avRoomUid;

  ///音视频服务器请求token
  final String avRoomCheckSum;

  ///会议创建时间，unix时间戳（单位：s）
  final int createTime;

  ///会议持续时间
  final int duration;

  /// 是否有全体禁音
  final int audioAllMute;

  /// 聊天室id, 创建参数chatRoom=1时返回, 为空表示改会议没有开启聊天室
  final String? chatRoomId;

  /// 推流地址: 创建参数live=1时返回, 为空表示改会议没有开启直播推流
  final String? pushUrl;

  /// 会议唯一key, 可用于获取直播参数
  //final String? meetingKey;

  /// nertc appKey
  final String nrtcAppKey;

  /// 会议录制开关，true开启，false关闭。
  final bool attendeeRecordOn;

  /// 白板账号
  final String? wbAccid;

  /// 白板token
  final String? wbToken;

  /// 白板key
  final String? wbKey;

  const JoinRoomInfo({
    required this.meetingId,
    required this.avRoomCName,
    required this.avRoomCid,
    required this.avRoomUid,
    required this.avRoomCheckSum,
    required this.createTime,
    required this.duration,
    required this.audioAllMute,
    this.chatRoomId,
    this.pushUrl,
    //this.meetingKey,
    required this.nrtcAppKey,
    required this.attendeeRecordOn,
    required this.wbAccid,
    required this.wbToken,
    required this.wbKey,
  });

  factory JoinRoomInfo.fromMap(Map map) {
    return JoinRoomInfo(
      meetingId: map['meetingId'] as String,
      avRoomCName: map['avRoomCName'] as String,
      avRoomCid: map['avRoomCid'] as String,
      avRoomUid: map['avRoomUid'] as int,
      avRoomCheckSum: map['avRoomCheckSum'] as String,
      createTime: (map['createTime'] ?? 0) as int,
      duration: (map['duration'] ?? 0) as int,
      audioAllMute: (map['audioAllMute'] ?? 0) as int,
      chatRoomId: (map['chatRoomId'] as int?)?.toString(),
      pushUrl: map['pushUrl'] as String?,
      //meetingKey: map['meetingKey'] as String?,
      nrtcAppKey: map['nrtcAppKey'] as String,
      attendeeRecordOn: (map['attendeeRecordOn'] ?? false) as bool,
      wbAccid: (map['wbAccid'] ?? '') as String,
      wbToken: (map['wbToken'] ?? '') as String,
      wbKey: (map['wbKey'] ?? '') as String,
    );
  }

  Map toMap() => {
        'meetingId': meetingId,
        'avRoomCName': avRoomCName,
        'avRoomCid': avRoomCid,
        'avRoomUid': avRoomUid,
        'avRoomCheckSum': avRoomCheckSum,
        'createTime': createTime,
        'duration': duration,
        'audioAllMute': audioAllMute,
        'chatRoomId': chatRoomId,
        'pushUrl': pushUrl,
        //'meetingKey': meetingKey,
        'nrtcAppKey': nrtcAppKey,
        'attendeeRecordOn': attendeeRecordOn,
        'wbAccid': wbAccid,
        'wbToken': wbToken,
        'wbKey': wbKey,
      };

  @override
  String toString() {
    return 'JoinRoomInfo{meetingId: $meetingId, avRoomCName: $avRoomCName, avRoomCid: $avRoomCid, avRoomUid: $avRoomUid, avRoomCheckSum: $avRoomCheckSum, createTime: $createTime, duration: $duration, audioAllMute: $audioAllMute, chatRoomId: $chatRoomId, pushUrl: $pushUrl, nrtcAppKey: $nrtcAppKey, attendeeRecordOn: $attendeeRecordOn, wbAccid: $wbAccid, wbToken: $wbToken, wbKey: $wbKey}';
  }
}
