// Copyright (c) 2014-2020 NetEase, Inc.
// All right reserved.

part of room_service;

/// 房间前直播信息
class NEPreRoomLiveInfo {

  final NERoomItemLive delegate;

  NEPreRoomLiveInfo() : delegate = NERoomItemLive();

  NEPreRoomLiveInfo.fromJson(Map? map)
      : delegate = NERoomItemLive.fromJson(map);

  /// 设置是否开启直播
  set enable(bool enable) => delegate.enable = enable;

  bool get enable => delegate.enable;

  /// 获取直播级别
  NELiveAuthLevel get webAccessControlLevel =>
      LiveAuthLevelExtension.get(delegate.liveWebAccessControlLevel);

  /// 设置直播级别
  set webAccessControlLevel(NELiveAuthLevel level) =>
      delegate.liveWebAccessControlLevel = level.index;

  String? get liveUrl => delegate.liveUrl;

  Map toJson() => {
    'enable': enable,
    'liveWebAccessControlLevel': webAccessControlLevel.value,
  };
}

/// 全量直播信息
///
class NERoomItemLive {
  /// 当前房间是否可以直播，在预约房间的时候需要打开允许直播开关
  bool _enable = false;

  set enable(bool enable) {
    _enable = enable;
  }

  bool get enable => _enable;

  ///登录 web 直播页的鉴权级别，0：不需要鉴权，1：需要登录，2：需要登录并且账号要与直播应用绑定。不填的话表示不需要鉴权
  int _liveWebAccessControlLevel = NELiveAuthLevel.token.index;

  int get liveWebAccessControlLevel => _liveWebAccessControlLevel;

  set liveWebAccessControlLevel(int liveWebAccessControlLevel) {
    _liveWebAccessControlLevel = liveWebAccessControlLevel;
  }

  /// 直播标题
  String? _title;

  set title(String? title) {
    _title = title;
  }

  String? get title => _title;

  /// 直播密码
  String? _password;

  set password(String? password) {
    _password = password;
  }

  String? get password => _password;

  ///是否使用独立的直播聊天室，如果不使用独立的直播聊天室，那么使用会议的聊天室作为直播聊天室
  bool _liveChatRoomIndependent = false;

  set liveChatRoomIndependent(bool liveChatRoomIndependent) {
    _liveChatRoomIndependent = liveChatRoomIndependent;
  }

  bool get liveChatRoomIndependent => _liveChatRoomIndependent;

  /// hls 拉流地址
  String? _hlsPullUrl;

  String? get hlsPullUrl => _hlsPullUrl;

  /// http 拉流地址
  String? _httpPullUrl;

  String? get httpPullUrl => _httpPullUrl;

  /// rtmp 拉流地址
  String? _rtmpPullUrl;

  String? get rtmpPullUrl => _rtmpPullUrl;

  /// 直播地址
  String? _liveUrl;

  String? get liveUrl => _liveUrl;

  /// 推流地址
  // String? _pushUrl;
  //
  // set pushUrl(String? pushUrl) {
  //   _pushUrl = pushUrl;
  // }
  //
  // String? get pushUrl => _pushUrl;

  /// 直播聊天室id
  String? _chatRoomId;

  String? get chatRoomId => _chatRoomId;

  /// 直播uid列表
  List<int>? _liveAVRoomUids;

  set liveAVRoomUids(List<int>? liveAVRoomUids) {
    _liveAVRoomUids = liveAVRoomUids;
  }

  List<int>? get liveAVRoomUids => _liveAVRoomUids;

  set liveLayout(int? liveLayout) {
    _liveLayout = liveLayout;
  }

  /// 视图模式
  int? _liveLayout;

  int? get liveLayout => _liveLayout;

  /// 直播web聊天室是否可用
  bool _liveChatRoomEnable = true;

  set liveChatRoomEnable(bool liveChatRoomEnable) {
    _liveChatRoomEnable = liveChatRoomEnable;
  }

  bool get liveChatRoomEnable => _liveChatRoomEnable;

  /// 会议id
  String? _roomId;

  set roomId(String? roomId) {
    _roomId = roomId;
  }

  String? get roomId => _roomId;

  /// 直播状态
  NERoomItemLiveState? _state;

  set state(NERoomItemLiveState? state) {
    _state = state;
  }

  NERoomItemLiveState? get state => _state;

  /// 任务id
  // String? _taskId;
  //
  // set taskId(String? taskId) {
  //   _taskId = taskId;
  // }
  //
  // String? get taskId => _taskId;

  Map toJson() => {
        'enable': enable,
        'liveWebAccessControlLevel': liveWebAccessControlLevel,
        if (_hlsPullUrl != null) 'hlsPullUrl': _hlsPullUrl,
        if (_httpPullUrl != null) 'httpPullUrl': _httpPullUrl,
        if (_rtmpPullUrl != null) 'rtmpPullUrl': _rtmpPullUrl,
        if (_liveUrl != null) 'liveUrl': _liveUrl,
        // if (_pushUrl != null) 'pushUrl': _pushUrl,
        if (_chatRoomId != null) 'chatRoomId': _chatRoomId,
        if (_liveAVRoomUids != null) 'liveAVRoomUids': _liveAVRoomUids,
        'liveChatRoomEnable': _liveChatRoomEnable,
        if (_roomId != null) 'meetingId': _roomId,
        if (_state != null) 'state': _state?.index,
        // if (_taskId != null) 'taskId': _taskId,
        if (_title != null) 'title': _title,
        if (_password != null) 'password': _password,
        'liveChatRoomIndependent': _liveChatRoomIndependent,
        'liveLayout': _liveLayout ?? LiveLayoutType.none,
      };

  NERoomItemLive();

  NERoomItemLive.fromJson(Map<dynamic, dynamic>? map) {
    if (map != null) {
      _enable = map['enable'] as bool;
      _liveWebAccessControlLevel =
      (map['liveWebAccessControlLevel'] ?? NELiveAuthLevel.token.index) as int;
      _hlsPullUrl = map['hlsPullUrl'] as String?;
      _httpPullUrl = map['httpPullUrl'] as String?;
      _rtmpPullUrl = map['rtmpPullUrl'] as String?;
      _liveUrl = map['liveUrl'] as String?;
      _chatRoomId = map['chatRoomId'] as String?;
      _liveAVRoomUids = (map['liveAVRoomUids'] as List?)?.cast<int>();
      _liveChatRoomEnable = (map['liveChatRoomEnable'] ?? true) as bool;
      _roomId = map['meetingId'] as String?;
      var _stateValue = (map['state'] ?? 0) as int;
      _state =
      _stateValue >= 0 && _stateValue < NERoomItemLiveState.values.length
          ? NERoomItemLiveState.values[_stateValue]
          : NERoomItemLiveState.invalid;
      _title = map['title'] as String?;
      _password = map['password'] as String?;
      _liveChatRoomIndependent =
      (map['liveChatRoomIndependent'] ?? false) as bool;
      _liveLayout = (map['liveLayout'] ?? LiveLayoutType.none) as int;
    }
  }

  NERoomItemLive copy() {
    final copy = NERoomItemLive();
    copy.._enable = _enable
    .._liveWebAccessControlLevel = _liveWebAccessControlLevel
      .._hlsPullUrl = _hlsPullUrl
      .._httpPullUrl = _httpPullUrl
      .._rtmpPullUrl = _rtmpPullUrl
      .._liveUrl = _liveUrl
      // .._pushUrl = _pushUrl
      .._chatRoomId = chatRoomId
      .._liveAVRoomUids = _liveAVRoomUids
      .._liveChatRoomEnable = _liveChatRoomEnable
      .._roomId = _roomId
      .._state = _state
      // .._taskId = _taskId
      .._title = _title
      .._password = _password
      .._liveChatRoomIndependent = _liveChatRoomIndependent
      .._liveLayout = _liveLayout;
    return copy;
  }

  @override
  bool operator ==(Object other) =>
      identical(this, other) ||
      other is NERoomItemLive &&
          runtimeType == other.runtimeType &&
          _enable == other._enable &&
          _liveWebAccessControlLevel == other._liveWebAccessControlLevel &&
          _title == other._title &&
          _password == other._password &&
          _hlsPullUrl == other._hlsPullUrl &&
          _httpPullUrl == other._httpPullUrl &&
          _rtmpPullUrl == other._rtmpPullUrl &&
          _liveUrl == other._liveUrl &&
          _chatRoomId == other._chatRoomId &&
          _liveAVRoomUids == other._liveAVRoomUids &&
          _liveLayout == other._liveLayout &&
          _liveChatRoomEnable == other._liveChatRoomEnable &&
          _state == other._state;

  @override
  int get hashCode =>
      _enable.hashCode ^
      _liveWebAccessControlLevel.hashCode ^
      _title.hashCode ^
      _password.hashCode ^
      _hlsPullUrl.hashCode ^
      _httpPullUrl.hashCode ^
      _rtmpPullUrl.hashCode ^
      _liveUrl.hashCode ^
      _chatRoomId.hashCode ^
      _liveAVRoomUids.hashCode ^
      _liveLayout.hashCode ^
      _liveChatRoomEnable.hashCode ^
      _state.hashCode;
}

enum NELiveLayout {
  none,
  gallery,
  focus,
  screenShare,
}
