// Copyright (c) 2014-2020 NetEase, Inc.
// All right reserved.

part of room_service;

class NERoomItemSettings {
  /// 参会者加入会议音频开启状态， false： 开启音频， true： 关闭音频
  /// 这个设置为 预约房间时的设置，房间内主持人全体静音/解除全体静音 不会修改该变量
  bool _attendeeAudioOff = false;

  bool get isAttendeeAudioOff => _attendeeAudioOff;

  set setAttendeeAudioOff(bool audioOff) {
    _attendeeAudioOff = audioOff;
  }

  /// 参会者加入会议云端录制开启状态， false： 开启， true： 关闭
  bool _cloudRecordOn = false;

  bool get cloudRecordOn => _cloudRecordOn;

  set cloudRecordOn(bool recordOn) {
    _cloudRecordOn = recordOn;
  }

  /// 参会者在全体静音状态下是否允许打开音频， false： 不允许， true： 允许
  late bool _allowSelfAudioOn = true;

  bool get isAllowSelfAudioOn => _allowSelfAudioOn;

  set allowSelfAudioOn(bool audioOn) {
    _allowSelfAudioOn = audioOn;
  }

  @override
  bool operator ==(Object other) =>
      identical(this, other) ||
      other is NERoomItemSettings &&
          runtimeType == other.runtimeType &&
          _attendeeAudioOff == other._attendeeAudioOff &&
          _cloudRecordOn == other._cloudRecordOn &&
          _scene == other._scene &&
          _allowSelfAudioOn == other._allowSelfAudioOn;

  @override
  int get hashCode =>
      _attendeeAudioOff.hashCode ^
      _cloudRecordOn.hashCode ^
      _scene.hashCode ^
      _allowSelfAudioOn.hashCode;

  ///会议角色配置信息
  NERoomScene? _scene;

  set scene(NERoomScene? scene) {
    _scene = scene;
  }

  Map toJson() => {
        'attendeeAudioOff': _attendeeAudioOff,
        'attendeeRecordOn': _cloudRecordOn,
        'allowSelfAudioOn': _allowSelfAudioOn,
        if(_scene != null) 'scene': _scene?.toJson(),
      };

  static NERoomItemSettings fromJson(Map<dynamic, dynamic>? map) {
    var setting = NERoomItemSettings();
    if (map != null) {
      setting.setAttendeeAudioOff = (map['attendeeAudioOff'] ?? false) as bool;
      setting.allowSelfAudioOn = (map['allowSelfAudioOn'] ?? true) as bool;
      setting.cloudRecordOn = (map['attendeeRecordOn'] ?? false) as bool;
      if(map['scene'] != null) {
        setting.scene = NERoomScene.fromMap(map['scene'] as Map);
      }
    }
    return setting;
  }
}
