// Copyright (c) 2014-2020 NetEase, Inc.
// All right reserved.

part of room_service;

abstract class HttpApi<T> extends BaseApi {
  static const _tag = 'HttpApi';
  @override
  Future<NEResult<T>> execute() async {
    var response = await _Executors().executeOverHttp(path(), header(), data());
    Alog.e(tag: _tag, moduleName: _moduleName, content: 'HttpApi>>>: path=${path()},header = ${header()}');
    if (response == null) {
      Alog.e(tag: _tag, moduleName: _moduleName, content: 'HttpApi: path=${path()}, result is null');
      return NEResult(code: RoomErrorCode.networkError);
    } else {
      if (response.statusCode == RoomErrorCode.success) {
        Alog.i(tag: _tag, moduleName: _moduleName, content:'HttpApi:resp path=${path()}, result: ${enableLog() ? response.data : response.statusCode}');
        if (response.data == null || !(response.data is Map)) {
          return NEResult(code: response.statusCode ?? RoomErrorCode.paramsError);
        } else {
          try {
            var map = response.data as Map;
            var code = map['code'] as int;
            if (code == RoomErrorCode.success) {
              var ret = map['ret'];
              return NEResult(code: RoomErrorCode.success, data: ret == null ? null : result(ret as Map) as T?);
            } else {
              RoomErrorRepository.httpApiError(code);
              return NEResult(code: code, msg: map['msg'] as String);
            }
          } catch (e, s) {
            Alog.e(tag: _tag, moduleName: _moduleName, content:'HttpApi: parse result data error: path=${path()}, exception=$e, stacktrace=\n$s');
            return NEResult(code: RoomErrorCode.serverError);
          }
        }
      } else {
        Alog.e(tag: _tag, moduleName: _moduleName, content:'HttpApi: path=$path code${response.statusCode}');
        RoomErrorRepository.httpApiError(response.statusCode ?? RoomErrorCode.paramsError);
        return NEResult(code: response.statusCode ?? RoomErrorCode.paramsError);
      }
    }
  }
}
