// Copyright (c) 2014-2020 NetEase, Inc.
// All right reserved.

part of room_service;

class IMMessageApi {
  IMMessageApi._internal();

  static final IMMessageApi _singleton = IMMessageApi._internal();

  factory IMMessageApi() => _singleton;

  StreamController<NotifyModel>? _streamController;

  Stream<NotifyModel> listen() {
    _streamController ??= StreamController.broadcast(
      onListen: _startListen,
      onCancel: _stopListen,
    );
    return _streamController!.stream;
  }

  void _notify(List<IMMessage> data) {
    Alog.d(
        tag: _tag,
        moduleName: _moduleName,
        content: 'IMMessageApi data = ${data.length}');
    data.forEach((message) {
      Alog.d(
          tag: _tag,
          moduleName: _moduleName,
          content: 'IMMessageApi message attach = ${message.attach}');
      _streamController?.add(NotifyModel(message.attach));
    });
  }

  void _startListen() {
    NimPlugin().getMsgObserver().registerReceiveMessage(_notify);
  }

  void _stopListen() {
    NimPlugin().getMsgObserver().unregisterReceiveMessage(_notify);
  }

  /// 发送控制消息
  static Future<int> sendMessage(String sessionId, AttachInfo info) {
    var message =
        MessageBuilder.createCustomMessage(sessionId, SessionType.p2p, info);
    Alog.d(
        tag: _tag,
        moduleName: _moduleName,
        content:
            'sendControlMessage sessionId = $sessionId, info = ${info.toMap()}');
    return NimApiHelper.sendMessage(message);
  }
}

class NotifyModel {
  String? content;
  NotifyModel(this.content);
}

class AttachInfo extends MsgAttachment {
  BaseData? data;

  /// must
  final int type;

  AttachInfo(this.type, {this.data});

  Map toMap() => {'type': type, if (data != null) 'data': data!.toMap()};

  @override
  String toJson() {
    return jsonEncode(toMap());
  }

  @override
  void fromJson(Map<dynamic, dynamic>? map) {}
}

abstract class BaseData {
  int type;
  int requestId;

  BaseData(this.type, this.requestId);

  Map toMap() {
    return {
      'type': type,
      'data': toData(),
    };
  }

  Map toData();
}
