// Copyright (c) 2014-2020 NetEase, Inc.
// All right reserved.

part of room_service;

/// passthrough base api
abstract class NimPassthroughApi<T> extends BaseApi {
  @override
  Future<NEResult<T>> execute() async {
    var resp = await _Executors().executeOverNIM(path(), header(), data());
    if (resp.data?.body == null) {
      Alog.e(
          tag: _tag,
          moduleName: _moduleName,
          content: 'NimPassthroughApi: path=${path()}, result is null');
      return NEResult(code: resp.code);
    }
    var content = jsonDecode(resp.data!.body!) as Map;
    if (content.isEmpty) {
      Alog.e(
          tag: _tag,
          moduleName: _moduleName,
          content: 'NimPassthroughApi: path=${path()}, content is null');
      return NEResult(code: resp.code);
    }
    try {
      var innerCode = content['code'] as int;
      if (innerCode == RoomErrorCode.success) {
        var ret = (content['ret'] ?? {}) as Map;
        return NEResult(code: innerCode, data: result(ret) as T?);
      } else {
        Alog.e(
            tag: _tag,
            moduleName: _moduleName,
            content: 'NimPassthroughApi: path=${path()}, code=$innerCode');
        RoomErrorRepository.nimApiError(innerCode);
        return NEResult(code: innerCode, msg: content['msg'] as String?);
      }
    } catch (e, s) {
      Alog.e(
          tag: _tag,
          moduleName: _moduleName,
          content:
              'NimPassthroughApi: parse result data error: path=${path()}, exception=$e, stacktrace=\n$s');
      if (kDebugMode) {
        rethrow;
      } else {
        return NEResult(code: RoomErrorCode.serverError);
      }
    }
  }
}
