// Copyright (c) 2014-2020 NetEase, Inc.
// All right reserved.

part of room_service;

class InRoomRepository {
  /// 移除参会人员
  static Future<NEResult<void>> hostRemoveMember(String meetingId, List members) =>
      NimApiHelper._hostControl(meetingId, ActionType.removeMember, members: members);

  /// 主持人禁止成员画面
  static Future<NEResult<void>> hostMuteVideo(String meetingId, List members) =>
      NimApiHelper._hostControl(meetingId, ActionType.hostMuteVideo, members: members);

  /// 主持人禁止成员声音
  static Future<NEResult<void>> hostMuteAudio(String meetingId, List members) =>
      NimApiHelper._hostControl(meetingId, ActionType.hostMuteAudio, members: members);

  /// 主持人全体禁音&允许用户自行解开
  static Future<NEResult<void>> hostMuteAllAudio(String meetingId, {List? members}) =>
      NimApiHelper._hostControl(meetingId, ActionType.hostMuteAllAudio, members: members);

  /// 主持人全体禁音&不允许用户自行解开
  static Future<NEResult<void>> hostMuteAllAudioOff(String meetingId, {List? members}) =>
      NimApiHelper._hostControl(meetingId, ActionType.hostMuteAllAudioOff, members: members);

  /// 主持人解禁成员画面
  static Future<NEResult<void>> hostUnMuteVideo(String meetingId, List members) =>
      NimApiHelper._hostControl(meetingId, ActionType.hostUnMuteVideo, members: members);

  /// 主持人解禁成员声音
  static Future<NEResult<void>> hostUnMuteAudio(String meetingId, List members) =>
      NimApiHelper._hostControl(meetingId, ActionType.hostUnMuteAudio, members: members);

  /// 主持人全体解禁
  static Future<NEResult<void>> hostUnMuteAllAudio(String meetingId, {List? members}) =>
      NimApiHelper._hostControl(meetingId, ActionType.hostUnMuteAllAudio, members: members);

  /// 移交主持人
  static Future<NEResult<void>> changeHost(String meetingId, List members) =>
      NimApiHelper._hostControl(meetingId, ActionType.changeHost, members: members);

  /// 主持人指定焦点视频
  static Future<NEResult<void>> hostSetFocusVideo(String meetingId, List members) =>
      NimApiHelper._hostControl(meetingId, ActionType.setFocusVideo, members: members);

  /// 主持人取消焦点视频
  static Future<NEResult<void>> hostCancelFocusVideo(String meetingId, List members) =>
      NimApiHelper._hostControl(meetingId, ActionType.cancelFocusVideo, members: members);

  /// 主持人锁定会议
  static Future<NEResult<void>> hostLockMeeting(String meetingId) =>
      NimApiHelper._hostControl(meetingId, ActionType.hostLockMeeting);

  /// 主持人解锁会议
  static Future<NEResult<void>> hostUnLockMeeting(String meetingId) =>
      NimApiHelper._hostControl(meetingId, ActionType.hostUnLockMeeting);

  /// 静音状态主持人拒绝成员举手
  static Future<NEResult<void>> hostRejectAudioHandsUp(String meetingId, List members) =>
      NimApiHelper._hostControl(meetingId, ActionType.hostRejectAudioHandsUp, members: members);


  /// 主持人结束屏幕共享
  static Future<NEResult<void>> hostStopScreenShare(String meetingId, List members) =>
      NimApiHelper._hostControl(meetingId, ActionType.hostStopScreenShare, members: members);

  /// 成员结束白板共享
  static Future<NEResult<void>> hostStopWhiteBoardShare(String meetingId, List members) =>
      NimApiHelper._hostControl(meetingId, ActionType.stopWhiteBoardShare, members: members);

  /// 成员开始白板共享
  static Future<NEResult<void>> hostStartWhiteBoardShare(String meetingId) =>
      NimApiHelper._hostControl(meetingId, ActionType.startWhiteBoardShare);

  /// 获取会议中的成员信息
  static Future<NEResult<MeetingInfo>> memberInfo(String meetingId, {int? avRoomUid}) =>
      NimApiHelper._getMemberInfo(meetingId, avRoomUid: avRoomUid);

  /// 服务器控制协议
  static Stream<MeetingAction> get onServerControlReceived => NimApiHelper._onServerControlReceived;

  /// 成员关闭自身画面
  static Future<NEResult<void>> muteSelfVideo(String meetingId) =>
      NimApiHelper._memberControl(meetingId, ActionType.muteSelfVideo);

  /// 成员关闭自身声音
  static Future<NEResult<void>> muteSelfAudio(String meetingId) =>
      NimApiHelper._memberControl(meetingId, ActionType.muteSelfAudio);

  /// 成员打开自身画面
  static Future<NEResult<void>> unMuteSelfVideo(String meetingId) =>
      NimApiHelper._memberControl(meetingId, ActionType.unMuteSelfVideo);

  /// 全体静音成员举手
  static Future<NEResult<void>> muteAllHandsUp(String meetingId) =>
      NimApiHelper._memberControl(meetingId, ActionType.muteAllHandsUp);

  /// 全体静音成员放手
  static Future<NEResult<void>> muteAllUnHandsUp(String meetingId) =>
      NimApiHelper._memberControl(meetingId, ActionType.muteAllUnHandsUp);

  /// 成员打开自身声音
  static Future<NEResult<void>> unMuteSelfAudio(String meetingId) =>
      NimApiHelper._memberControl(meetingId, ActionType.unMuteSelfAudio);

  /// 成员结束屏幕共享
  static Future<NEResult<void>> stopScreenShare(String meetingId) =>
      NimApiHelper._memberControl(meetingId, ActionType.stopScreenShare);

  /// 成员开始屏幕共享
  static Future<NEResult<void>> startScreenShare(String meetingId) =>
      NimApiHelper._memberControl(meetingId, ActionType.startScreenShare);

  /// 成员结束白板共享
  static Future<NEResult<void>> stopWhiteBoardShare(String meetingId) =>
      NimApiHelper._memberControl(meetingId, ActionType.stopWhiteBoardShare);

  /// 成员开始白板共享
  static Future<NEResult<void>> startWhiteBoardShare(String meetingId) =>
      NimApiHelper._memberControl(meetingId, ActionType.startWhiteBoardShare);

  /// 授予成员白板互动权限
  static Future<NEResult<void>> awardedMemberWhiteboardInteraction(String meetingId, List avRoomUids) =>
      NimApiHelper._memberControl(meetingId, ActionType.awardedMemberWhiteboardInteraction,avRoomUids: avRoomUids);

  ///撤销成员白板互动权限
  static Future<NEResult<void>> undoMemberWhiteboardInteraction(String meetingId, List avRoomUids) =>
      NimApiHelper._memberControl(meetingId, ActionType.undoMemberWhiteboardInteraction, avRoomUids: avRoomUids);

  static Future<NEResult<void>> modifyNickname(String meetingId, String nickname) =>
      NimApiHelper._modifyNickname(meetingId, nickname);
}
