// Copyright (c) 2014-2020 NetEase, Inc.
// All right reserved.

part of room_service;

class RoomRepository {
  /// 创建会议
  static Future<NEResult<JoinRoomInfo>> createRoom(
      String nickName,
      String? meetingId,
      String? password,
      String? tag,
      int video,
      int audio,
      bool enableChatroom,
      bool attendeeRecordOn,
      Map? scene) {
    UserProfile.displayName = nickName;
    return HttpApiHelper._createMeeting(
        nickName, meetingId, password, tag, video, audio, enableChatroom, attendeeRecordOn, scene);
  }

  /// 加入会议
  static Future<NEResult<JoinRoomInfo>> joinRoom(String nickName, String meetingId, String? tag,int video, int audio, String? password) {
    UserProfile.displayName = nickName;
    return HttpApiHelper._joinMeeting(nickName, meetingId, tag, video, audio, password);
  }

  /// 匿名加入会议
  static Future<NEResult<AnonymousJoinMeetingInfo>> anonymousJoinRoom(
      String meetingId, String nickName,String? tag , int video, int audio, String? password) async {
    UserProfile.displayName = nickName;
    var info = await HttpApiHelper._anonymousJoinMeeting(meetingId, nickName, tag, video, audio, password);
    var completer = Completer<NEResult<AnonymousJoinMeetingInfo>>();
    if (info.code == RoomErrorCode.success && info.data != null) {
      final annoyData = info.nonNullData;
      var annoyLoginInfo = _LoginResponse(
        accountId: annoyData.accountId,
        nickname: annoyData.nickName,
        accountToken: annoyData.accountToken,
        appKey: annoyData.appKey,
        imAccid: annoyData.imAccid,
        imToken: annoyData.imToken,
        imAppKey: annoyData.imAppKey,
        nrtcAppKey: annoyData.nrtcAppKey,
      );
      await _AuthManager()._imLoginCheckWithLogin(loginResponse: annoyLoginInfo).then((ret) {
        if (completer.isCompleted) {
          return;
        }
        if (ret.code == RoomErrorCode.success) {
          completer.complete(info);
        } else {
          completer.complete(NEResult(code: ret.code, msg: ret.msg));
        }
      });
    } else {
      completer.complete(NEResult(code: info.code, msg: RoomErrorCode.getMsg(info.msg)));
    }
    return completer.future;
  }

  /// 关闭会议
  static Future<NEResult<void>> closeRoom(String roomId) => NimApiHelper._closeRoom(roomId);
}
