// Copyright (c) 2014-2020 NetEase, Inc.
// All right reserved.

part of room_service;

class SettingsRepository {
  static int? _beautyLevelCache;

  static int? get beautyLevelCache => _beautyLevelCache;
  static Uint8List? _beautyKeyCache;

  ///保存用户信息配置
  static Future<NEResult<void>> saveSettingsApi(BeautySettings status) async {
    return await HttpApiHelper._saveSettingsApi(status);
  }

  ///获取用户信息配置
  static Future<NEResult<AccountSettings>> getSettingsApi() async {
    return HttpApiHelper._getSettingsApi();
  }

  /// 获取用户信息配置
  static Future<NEResult<Uint8List>?> getBeautyLicenseApi() async {
    NEResult<Uint8List> result;
    ProgressCallback progressCallback = (int count, int total) async {
    };
    var url =  (SDKConfig.beauty.config as BeautyConfig?)?.licenseUrl;
    if(url == null){
      return Future.value(null);
    }
    if(!isBeautyFaceSupported()) return null;
    if(_beautyKeyCache !=null){
      result = NEResult<Uint8List>(code:RoomErrorCode.success, data: _beautyKeyCache);
    }else{
      result = (await  HttpApiHelper._getBeautyLicenseApi(
          url,  progressCallback));
      _beautyKeyCache = result.data;
    }
    return result;
  }

  /// 美颜等级在[0-10]范围
  static Future<NEResult<void>> setBeautyFaceValue(int value) async {
    _beautyLevelCache = value;
    return await SettingsRepository.saveSettingsApi(
        BeautySettings(beauty: Beauty(level: value)));
  }

  static Future<NEResult<int>> getBeautyFaceValue() async {
    NEResult<int> result;
    if (AuthRepository.isAuthorized()) {
      if (_beautyLevelCache != null) {
        result = NEResult.successWith(_beautyLevelCache!);
      }
      var beautySettings = await SettingsRepository.getSettingsApi();
      var level = beautySettings.data?.settings?.beauty.level ?? 0;
      if (beautySettings.isSuccess()) {
        result = NEResult.successWith(level);
      } else {
        result = NEResult(code: beautySettings.code, data: level);
      }
    } else {
      result = NEResult(code: RoomErrorCode.tokenError,data: 0);
    }
    return result;
  }

  static bool isBeautyFaceSupported()  {
    return ((SDKConfig.beauty.status == 1) && (SDKConfig.beauty.config as BeautyConfig?)?.licenseUrl != null);
  }

  static bool isMeetingLiveSupported()  {
    return SDKConfig.live.status == 1;
  }

  static bool isMeetingWhiteboardSupported()  {
    return SDKConfig.whiteboard.status == 1;
  }

  static bool isMeetingCloudRecordSupported()  {
    return SDKConfig.meetingRecord.status == 1;
  }

  static bool isMeetingChatSupported()  {
    return SDKConfig.meetingChat.status == 1;
  }


  /// 会议音频录制开启，true：开启，false：关闭，true由服务器抄送给应用的回调接口
  bool get meetingRecordAudioEnable => SDKConfig.meetingRecordAudioEnable;

  /// 会议视频录制开启，true：开启，false：关闭，true由服务器抄送给应用的回调接口
  bool get meetingRecordVideoEnable => SDKConfig.meetingRecordVideoEnable;

  /// 会议录制模式，0：混合与单人，1：混合，2：单人，但只在有值的时候，才会由服务器抄送给应用的回调接口
  int get meetingRecordMode => SDKConfig.meetingRecordMode;

  static Future<void> clear() async {
    _beautyLevelCache = null;
    _beautyKeyCache = null;
  }
}
